@@ -1,26 +1,28 @@
 #!/bin/sh
 
 # Command history plugin via FZF for CliFM
+# Dependencies: fzf
+
 # Written by L. Abramovich
 # License GPL3
 
-if [ -n "$1" ] && { [ "$1" = "--help" ] || [ "$1" = "help" ]; }; then
-	name="$(basename "$0")"
+if [ -n "$1" ] && { [ "$1" = "--help" ] || [ "$1" = "-h" ]; }; then
+	name="${CLIFM_PLUGIN_NAME:-$(basename "$0")}"
 	printf "Navigate and execute CliFM commands history via FZF\n"
-	printf "Usage: %s\n" "$name"
+	printf "\n\x1b[1mUSAGE\x1b[0m\n  %s\n" "$name"
 	exit 0
 fi
 
 if ! type fzf > /dev/null 2>&1; then
-	printf "CliFM: fzf: Command not found" >&2
-	exit 1
+	printf "clifm: fzf: Command not found" >&2
+	exit 127
 fi
 
-FILE="${XDG_CONFIG_HOME:=$HOME/.config}/clifm/profiles/$CLIFM_PROFILE/history.cfm"
+FILE="${XDG_CONFIG_HOME:=$HOME/.config}/clifm/profiles/$CLIFM_PROFILE/history.clifm"
 
 # Source our plugins helper
 if [ -z "$CLIFM_PLUGINS_HELPER" ] || ! [ -f "$CLIFM_PLUGINS_HELPER" ]; then
-	printf "CliFM: Unable to find plugins-helper file\n" >&2
+	printf "clifm: Unable to find plugins-helper file\n" >&2
 	exit 1
 fi
 # shellcheck source=/dev/null
