@@ -2,7 +2,7 @@
  *
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -26,7 +26,7 @@
 
 __BEGIN_DECLS
 
-void parse_autocmd_line(char *);
+void parse_autocmd_line(char *cmd, const size_t buflen);
 void reset_opts(void);
 int  check_autocmds(void);
 void revert_autocmd_opts(void);
