@@ -1011,252 +1011,6 @@ construct_human_size(const off_t size)
 	return str;
 }
 
-#ifdef USE_XDU
-struct hlink_t {
-	dev_t dev;
-	ino_t ino;
-};
-
-static struct hlink_t *xdu_hardlinks = {0};
-static size_t xdu_hardlink_n = 0;
-
-static inline int
-check_xdu_hardlinks(const dev_t dev, const ino_t ino)
-{
-	if (!xdu_hardlinks || xdu_hardlink_n == 0)
-		return 0;
-
-	size_t i;
-	for (i = 0; i < xdu_hardlink_n; i++) {
-		if (dev == xdu_hardlinks[i].dev && ino == xdu_hardlinks[i].ino)
-			return 1;
-	}
-
-	return 0;
-}
-
-static inline void
-add_xdu_hardlink(const dev_t dev, const ino_t ino)
-{
-	xdu_hardlinks = xnrealloc(xdu_hardlinks, xdu_hardlink_n + 1,
-		sizeof(struct hlink_t));
-
-	xdu_hardlinks[xdu_hardlink_n].dev = dev;
-	xdu_hardlinks[xdu_hardlink_n].ino = ino;
-	xdu_hardlink_n++;
-}
-
-static inline void
-free_xdu_hardlinks(void)
-{
-	free(xdu_hardlinks);
-	xdu_hardlinks = (struct hlink_t *)NULL;
-	xdu_hardlink_n = 0;
-}
-
-/* The st_size member of a stat struct is meaningful only:
- * 1. When computing disk usage (not apparent sizes).
- * 2. If apparent sizes, only for symlinks and regular files.
- * NOTE: Here we add shared memory object and typed memory object just to
- * match the check made by du(1). These objects are not implmented on most
- * systems, but this might change in the future. */
-static inline int
-usable_st_size(struct stat const *s)
-{
-	return (conf.apparent_size != 1 || S_ISLNK(s->st_mode)
-		|| S_ISREG(s->st_mode) || S_TYPEISSHM(s) || S_TYPEISTMO(s));
-}
-
-/* Read no more than this many directory entries at a time. Without this
- * limit, processing a directory with 4,000,000 entries requires ~1GiB of
- * memory, and handling 64M entries would require 16GiB of memory. */
-//#define MAX_READDIR_ENTRIES 100000
-
-/* Hand-made implementation of du(1) providing only those features required
- * by clifm.
- *
- * Returns the full size of the directory DIR in bytes, computing apparent
- * sizes, if conf.apparent_size is set to 1, or disk usage otherwise. In case
- * of error, STATUS is set to the appropiate error code (errno): in this case,
- * the return value may be zero, if DIR couldn't be read for some reason, or
- * a positive integer representing the size of the entries that have been
- * actually processed.
- *
- * FIRST_LEVEL must be always 1 when calling this function (this value will
- * be zero whenever the function calls itself recursively).
- *
- * NOTE: Old versions of du (at least up to 8.30) count the size of directories
- * themselves when computing apparent sizes. At least since 9.3, this is not
- * the case anymore (See
- * https://git.savannah.gnu.org/gitweb/?p=coreutils.git;a=commit;h=110bcd28386b1f47a4cd876098acb708fdcbbb25).
- * As stated in 'info du': "Apparent sizes are meaningful only for regular
- * files and symbolic links. Other file types do not contribute to apparent
- * size." We follow here the last behavior. */
-off_t
-dir_size(char *dir, const int first_level, int *status)
-{
-	if (!dir || !*dir) {
-		*status = ENOENT;
-		return 0;
-	}
-
-	struct stat a;
-//	const dev_t base_dev = lstat(dir, &a) != -1 ? a.st_dev : 0;
-	off_t size = 0;
-	DIR *p;
-
-	if ((p = opendir(dir)) == NULL) {
-		*status = errno;
-		return 0;
-	}
-
-	/* Compute the size of the base directory itself */
-	if (conf.apparent_size != 1 && first_level == 1
-	&& stat(dir, &a) != -1)
-		size += (a.st_blocks * S_BLKSIZE);
-
-	struct dirent *ent;
-
-	while ((ent = readdir(p)) != NULL) {
-		if (SELFORPARENT(ent->d_name))
-			continue;
-
-		char buf[PATH_MAX + 1];
-		snprintf(buf, sizeof(buf), "%s/%s", dir, ent->d_name);
-
-		if (lstat(buf, &a) == -1) {
-			*status = errno;
-			continue;
-		}
-
-/*		if (base_dev > 0 && base_dev != a.st_dev) {
-			*status = EXDEV; // File is on a different file system.
-			continue;
-		} */
-
-		if (S_ISDIR(a.st_mode)) {
-			/* Even if a subdirectory is unreadable or we can't chdir into
-			 * it, do let its size contribute to the total (provided we're
-			 * not computing apparent sizes). */
-			if (conf.apparent_size != 1)
-				size += (a.st_blocks * S_BLKSIZE);
-
-			if (user.uid != 0
-			&& check_file_access(a.st_mode, a.st_uid, a.st_gid) == 0)
-				*status = EACCES;
-			else
-				size += dir_size(buf, 0, status);
-
-			continue;
-		}
-
-		if (usable_st_size(&a) == 0)
-			continue;
-
-		if (a.st_nlink > 1) {
-			if (check_xdu_hardlinks(a.st_dev, a.st_ino) == 1)
-				continue;
-			else
-				add_xdu_hardlink(a.st_dev, a.st_ino);
-		}
-
-		size += conf.apparent_size == 1 ? a.st_size
-			: (a.st_blocks * S_BLKSIZE);
-	}
-
-	closedir(p);
-
-	if (first_level == 1)
-		free_xdu_hardlinks();
-
-	return size;
-}
-#else
-/* Return the full size of the directory DIR using du(1).
- * The size is reported in bytes if SIZE_IN_BYTES is set to 1.
- * Otherwise, human format is used.
- * STATUS is updated to the command exit code. */
-off_t
-dir_size(char *dir, const int size_in_bytes, int *status)
-{
-	if (!dir || !*dir)
-		return (-1);
-
-	char file[PATH_MAX + 1];
-	snprintf(file, sizeof(file), "%s/%s", xargs.stealth_mode == 1
-		? P_tmpdir : tmp_dir, TMP_FILENAME); /* NOLINT */
-
-	int fd = mkstemp(file);
-	if (fd == -1)
-		return (-1);
-
-	int stdout_bk = dup(STDOUT_FILENO); /* Save original stdout */
-	if (stdout_bk == -1) {
-		unlinkat(fd, file, 0);
-		close(fd);
-		return (-1);
-	}
-
-	/* Redirect stdout to the desired file */
-	dup2(fd, STDOUT_FILENO);
-	close(fd);
-
-	if (bin_flags & (GNU_DU_BIN_DU | GNU_DU_BIN_GDU)) {
-		char *block_size = (char *)NULL;
-
-		if (size_in_bytes == 1) {
-			block_size = "--block-size=1";
-		} else {
-			if (xargs.si == 1)
-				block_size = "--block-size=KB";
-			else
-				block_size = "--block-size=K";
-		}
-
-		char *bin = (bin_flags & GNU_DU_BIN_DU) ? "du" : "gdu";
-		if (conf.apparent_size != 1) {
-			char *cmd[] = {bin, "-s", block_size, "--", dir, NULL};
-			*status = launch_execv(cmd, FOREGROUND, E_NOSTDERR);
-		} else {
-			char *cmd[] = {bin, "-s", "--apparent-size", block_size,
-				"--", dir, NULL};
-			*status = launch_execv(cmd, FOREGROUND, E_NOSTDERR);
-		}
-	} else {
-		char *cmd[] = {"du", "-ks", "--", dir, NULL};
-		*status = launch_execv(cmd, FOREGROUND, E_NOSTDERR);
-	}
-
-	dup2(stdout_bk, STDOUT_FILENO); /* Restore original stdout */
-	close(stdout_bk);
-
-	FILE *fp = open_fread(file, &fd);
-	if (!fp) {
-		unlink(file);
-		return (-1);
-	}
-
-	off_t retval = -1;
-	/* We only need here the first field of the line, which is a file
-	 * size and usually takes only a few digits: since a yottabyte takes 26
-	 * digits, 32 is more than enough. */
-	char line[32]; *line = '\0';
-	if (fgets(line, (int)sizeof(line), fp) == NULL)
-		goto END;
-
-	char *p = strchr(line, '\t');
-	if (p && p != line) {
-		*p = '\0';
-		retval = (off_t)atoll(line);
-	}
-
-END:
-	unlinkat(fd, file, 0);
-	fclose(fp);
-	return retval;
-}
-#endif /* USE_XDU */
-
 /* Return the file type of the file pointed to by LINK, or -1 in case of
  * error. Possible return values:
 S_IFDIR: 40000 (octal) / 16384 (decimal, integer)
