@@ -899,6 +899,7 @@ reset_iface_colors(void)
 	*wm_c = '\0';
 	*wp_c = '\0';
 	*xf_c = '\0';
+	*xf_cb = '\0';
 	*xs_c = '\0';
 
 	*ws1_c = '\0';
@@ -1407,8 +1408,10 @@ set_iface_colors(char **colors, const size_t num_colors)
 		else if (*colors[i] == 'x') {
 			if (colors[i][1] == 's' && colors[i][2] == '=')
 				set_color(colors[i] + 3, xs_c, RL_NO_PRINTABLE);
-			else if (colors[i][1] == 'f' && colors[i][2] == '=')
+			else if (colors[i][1] == 'f' && colors[i][2] == '=') {
 				set_color(colors[i] + 3, xf_c, RL_NO_PRINTABLE);
+				set_color(colors[i] + 3, xf_cb, RL_PRINTABLE);
+			}
 		}
 
 		free(colors[i]);
@@ -1522,11 +1525,15 @@ set_default_size_shades(void)
 static int
 store_extension_line(const char *line)
 {
-	/* Remove the leading "*." from the extension line */
-	if (!line || *line != '*' || *(line + 1) != '.' || !*(line + 2))
+	if (!line || !*line)
 		return FUNC_FAILURE;
 
-	line += 2;
+	/* Remove the leading "*.", if any, from the extension line. */
+	if (*line == '*' && *(line + 1) == '.') {
+		line += 2;
+		if (!*line)
+			return FUNC_FAILURE;
+	}
 
 	char *q = strchr(line, '=');
 	if (!q || !*(q + 1) || q == line)
@@ -1602,7 +1609,8 @@ check_ext_color_hash_conflicts(void)
 static void
 split_extension_colors(char *extcolors)
 {
-	char *p = extcolors, *buf = (char *)NULL;
+	char *p = extcolors;
+	char *buf = (char *)NULL;
 	size_t len = 0;
 	int eol = 0;
 
@@ -1615,7 +1623,10 @@ split_extension_colors(char *extcolors)
 		case '\n': /* fallthrough */
 		case ':':
 			if (!buf || !*buf) {
-				eol = 1;
+				if (!*p || !*(p + 1))
+					eol = 1;
+				else
+					p++;
 				break;
 			}
 
@@ -1731,6 +1742,7 @@ set_default_colors_256(void)
 	if (!*wc_c) xstrsncpy(wc_c, DEF_WC_C256, sizeof(wc_c));
 	if (!*xs_c) xstrsncpy(xs_c, DEF_XS_C256, sizeof(xs_c));
 	if (!*xf_c) xstrsncpy(xf_c, DEF_XF_C256, sizeof(xf_c));
+	if (!*xf_cb) xstrsncpy(xf_cb, DEF_XF_CB256, sizeof(xf_cb));
 
 	/* File types */
 	if (!*bd_c) xstrsncpy(bd_c, DEF_BD_C256, sizeof(bd_c));
@@ -1902,6 +1914,7 @@ set_default_colors(void)
 	if (!*ws8_c) xstrsncpy(ws8_c, DEF_WS8_C, sizeof(ws8_c));
 	if (!*xs_c) xstrsncpy(xs_c, DEF_XS_C, sizeof(xs_c));
 	if (!*xf_c) xstrsncpy(xf_c, DEF_XF_C, sizeof(xf_c));
+	if (!*xf_cb) xstrsncpy(xf_cb, DEF_XF_CB, sizeof(xf_cb));
 
 	/* File types */
 	if (!*bd_c) xstrsncpy(bd_c, DEF_BD_C, sizeof(bd_c));
@@ -2201,6 +2214,10 @@ set_fzf_opts(char *line)
 
 	if (strstr(conf.fzftab_options, "--height"))
 		fzf_height_set = 1;
+
+	char *b = strstr(conf.fzftab_options, "--border");
+	if (b)
+		set_fzf_border_type(b + (sizeof("--border") - 1));
 }
 
 static void
@@ -2259,7 +2276,28 @@ set_cs_extcolors(char *line, char **extcolors, const ssize_t line_len)
 		l--;
 	}
 
-	*extcolors = savestring(p, (size_t)l);
+	if (!*extcolors) { /* First ExtColors line. */
+		*extcolors = savestring(p, (size_t)l);
+	} else {
+		/* Second ExtColors line or more: append it to the first one. */
+		size_t curlen = strlen(*extcolors);
+		if (curlen > 0 && (*extcolors)[curlen - 1] == ':') {
+			(*extcolors)[curlen - 1] = '\0';
+			curlen--;
+		}
+
+		if (*p == ':' && *(p + 1) && l > 0) {
+			p++;
+			l--;
+		}
+
+		const size_t total_len = curlen + (size_t)l + 1;
+		*extcolors = xnrealloc(*extcolors, total_len + 1, sizeof(char *));
+
+		*(*extcolors + curlen) = ':';
+		memcpy(*extcolors + curlen + 1, p, (size_t)l);
+		(*extcolors)[total_len] = '\0';
+	}
 }
 
 #ifndef _NO_ICONS
@@ -2398,7 +2436,7 @@ read_color_scheme_file(const char *colorscheme, char **filecolors,
 		}
 
 		/* File extension colors */
-		else if (!*extcolors && *line == 'E'
+		else if (xargs.lscolors == UNSET && *line == 'E'
 		&& strncmp(line, "ExtColors=", 10) == 0) {
 			set_cs_extcolors(line, extcolors, line_len);
 		}
@@ -2594,6 +2632,7 @@ disable_bold(void)
 	remove_bold_attr(tx_c);
 	remove_bold_attr(xs_c);
 	remove_bold_attr(xf_c);
+	remove_bold_attr(xf_cb);
 
 	remove_bold_attr(wm_c);
 }
