@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,27 +25,11 @@
 #ifndef EXEC_H
 #define EXEC_H
 
-#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__sun)
-# include <signal.h>
-#endif /* BSD || __sun */
-
-#define EXEC_BG_PROC 0
-#define EXEC_FG_PROC 1
-
-#define EXEC_NULLPARAM EXIT_FAILURE
-#define EXEC_NOTFOUND  127
-#define EXEC_SIGINT    128
-
-#define NOTFOUND_MSG "Command not found"
-
 __BEGIN_DECLS
 
-int  exec_cmd(char **);
-void exec_chained_cmds(char *);
+int  exec_cmd(char **comm);
+void exec_chained_cmds(char *cmd);
 void exec_profile(void);
-int  get_exit_code(const int, const int);
-int  launch_execv(char **, const int, const int);
-int  launch_execl(const char *);
 
 __END_DECLS
 
