@@ -889,7 +889,7 @@ err_no_link(const char *file)
 	if (tlen != -1)
 		target[tlen] = '\0';
 
-	xerror(_("open: '%s': Broken symbolic link to %s\n"), file,
+	xerror(_("open: '%s': Broken symbolic link to '%s'\n"), file,
 		*target ? target : "???");
 
 	return saved_errno;
@@ -1174,10 +1174,19 @@ symlink_file(char **args)
 	char *target = args[0];
 	char *link_name = args[1];
 	char tmp[PATH_MAX + 1];
+	struct stat a;
 
 	if (!link_name || !*link_name) {
 		char *p = strrchr(target, '/');
 		snprintf(tmp, sizeof(tmp), "%s.link", (p && p[1]) ? p + 1 : target);
+
+		int suffix = 1;
+		while (lstat(tmp, &a) == 0 && suffix < INT_MAX) {
+			snprintf(tmp, sizeof(tmp), "%s.link-%d",
+				(p && p[1]) ? p + 1 : target, suffix);
+			suffix++;
+		}
+
 		link_name = tmp;
 	}
 
@@ -1185,7 +1194,6 @@ symlink_file(char **args)
 	if (len > 1 && link_name[len - 1] == '/')
 		link_name[len - 1] = '\0';
 
-	struct stat a;
 	if (lstat(target, &a) == -1) {
 		printf("link: '%s': %s\n", target, strerror(errno));
 		if (rl_get_y_or_n(_("Create broken symbolic link? [y/n] ")) == 0)
@@ -1198,14 +1206,13 @@ symlink_file(char **args)
 			return FUNC_SUCCESS;
 
 		if (unlinkat(XAT_FDCWD, link_name, 0) == -1) {
-			xerror("link: unlink: '%s': %s\n", link_name, strerror(errno));
+			xerror(_("link: Cannot unlink '%s': %s\n"),
+				link_name, strerror(errno));
 			return FUNC_FAILURE;
 		}
 	}
 
-	const int ret = symlinkat(target, XAT_FDCWD, link_name);
-
-	if (ret == -1) {
+	if (symlinkat(target, XAT_FDCWD, link_name) == -1) {
 		xerror(_("link: Cannot create symbolic link '%s': %s\n"),
 			link_name, strerror(errno));
 		return FUNC_FAILURE;
@@ -1537,12 +1544,28 @@ cp_mv_file(char **args, const int copy_and_rename, const int force)
 	return FUNC_SUCCESS;
 }
 
+/* Print the file name FNAME, quoted if it contains an space.
+ * A slash is appended if FNAME is a directory (ISDIR >= 1). */
+static void
+print_file_name(char *fname, const int isdir)
+{
+	if (strchr(fname, ' ')) {
+		if (strchr(fname, '\''))
+			printf("\"%s%s\"\n", fname, isdir >= 1 ? "/" : "");
+		else
+			printf("'%s%s'\n", fname, isdir >= 1 ? "/" : "");
+	} else {
+		fputs(fname, stdout);
+		puts(isdir >= 1 ? "/" : "");
+	}
+}
+
 static void
 print_removed_file_info(const struct rm_info info)
 {
 	char *p = abbreviate_file_name(info.name);
-	fputs(p ? p : info.name, stdout);
-	puts(info.dir >= 1 ? "/" : "");
+
+	print_file_name(p ? p : info.name, info.dir);
 
 	/* Name removed, but file is still linked to another name (hardlink) */
 	if (info.dir == 0 && info.links > 1) {
@@ -1596,12 +1619,8 @@ rm_confirm(struct rm_info *info, const size_t start, const int have_dirs)
 		have_dirs == 1 ? _(" (recursively)") : "");
 
 	size_t i;
-	for (i = start; info[i].name; i++) {
-		if (strchr(info[i].name, '\''))
-			printf("\"%s%s\"\n", info[i].name, info[i].dir >= 1 ? "/" : "");
-		else
-			printf("'%s%s'\n", info[i].name, info[i].dir >= 1 ? "/" : "");
-	}
+	for (i = start; info[i].name; i++)
+		print_file_name(info[i].name, info[i].dir);
 
 	return rl_get_y_or_n(_("Continue? [y/n] "));
 }
