@@ -28,8 +28,8 @@
 #define PROGRAM_NAME_UPPERCASE "CliFM"
 #define PROGRAM_NAME "clifm"
 #define PROGRAM_DESC "The command line file manager"
-#define VERSION "1.18"
-#define DATE "Mar 23, 2024"
+#define VERSION "1.19"
+#define DATE "Jun 18, 2024"
 #define AUTHOR "L. Abramovich"
 #define CONTACT "https://github.com/leo-arch/clifm"
 #define LICENSE "GPL2+"
@@ -112,11 +112,13 @@
 # define _NO_MAGIC
 #endif /* _NO_LIRA && !_NO_MAGIC */
 
-#if (defined(__linux__) || defined(__CYGWIN__) || defined(__HAIKU__)) \
+#if defined(USE_DU1)
+# if (defined(__linux__) || defined(__CYGWIN__) || defined(__HAIKU__)) \
 && !defined(_BE_POSIX)
 /* du(1) can report sizes in bytes, apparent sizes, and take custom block sizes */
-# define HAVE_GNU_DU
-#endif /* (__linux__ || __CYGWIN__ || __HAIKU__) && !_BE_POSIX */
+#  define HAVE_GNU_DU
+# endif /* (__linux__ || __CYGWIN__ || __HAIKU__) && !_BE_POSIX */
+#endif /* USE_DU1 */
 
 #ifndef _NO_GETTEXT
 # include <libintl.h>
@@ -515,9 +517,12 @@ extern time_t curdir_mtime;
 #define FZF_BIN_OK     (1 << 0)
 #define FNF_BIN_OK     (1 << 1)
 #define SMENU_BIN_OK   (1 << 2)
-#define GNU_DU_BIN_DU  (1 << 3)
-/* 'gdu' is the GNU version of 'du' used by BSD systems */
-#define GNU_DU_BIN_GDU (1 << 4)
+#ifdef USE_DU1
+ #define GNU_DU_BIN_DU  (1 << 3)
+ /* 'gdu' is the GNU version of 'du' used by BSD systems */
+ #define GNU_DU_BIN_GDU (1 << 4)
+#endif /* USE_DU1 */
+
 /* In BSD/Solaris systems, GNU utilities are provided as gcp, gmv, grm,
  * gdu, etc.
  * NOTE: Both FreeBSD and DragonFly native utilities provide all the
@@ -1367,7 +1372,7 @@ extern struct opts_t workspace_opts[MAX_WS];
  * line, to avoid overriding them with init_config(). While no command
  * line parameter will be overriden, the user still can modifiy on the
  * fly (editing the config file) any option not specified in the command
- * line */
+ * line. */
 struct param_t {
 	int apparent_size;
 	int auto_open;
@@ -1593,7 +1598,7 @@ struct props_t {
 	int ids; /* User/group IDs: either NUMBER or NAME */
 	int inode; /* File inode number */
 	int len; /* Approx len of the entire properties string taking into account
-			  * the all fields and their length. */
+			  * all fields and their length. */
 	int links; /* File links */
 	int blocks;
 	int no_group; /* Should we display group if IDS is set? */
@@ -1668,6 +1673,15 @@ struct ext_mnt_t {
 extern struct ext_mnt_t *ext_mnt;
 #endif /* LINUX_FSINFO */
 
+struct dir_info_t {
+	unsigned long long dirs;
+	unsigned long long files;
+	unsigned long long links;
+	off_t size;
+	int status;
+	int pad0;
+};
+
 enum tab_mode {
 	STD_TAB =   0,
 	FZF_TAB =   1,
@@ -1771,6 +1785,7 @@ extern int
 	exit_code,
 	follow_symlinks,
 	fzftab,
+	fzf_border_type,
 	fzf_height_set,
 	fzf_preview_border_type,
 	hist_status,
@@ -1822,7 +1837,6 @@ extern size_t
 	current_hist_n,
 	curhistindex,
 	ext_colors_n,
-//	files, // Amount of files in the current dir. Is size_t enough?
 	jump_n,
 	kbinds_n,
 	longest,
@@ -2046,6 +2060,7 @@ extern char
 	wm_c[MAX_COLOR + 2], /* Warning msg */
 	xs_c[MAX_COLOR + 2], /* Exit code: success */
 	xf_c[MAX_COLOR + 2], /* Exit code: failure */
+	xf_cb[MAX_COLOR],    /* Exit code: failure (dir read) */
 
 	tmp_color[MAX_COLOR + 2], /* A temp buffer to store color codes */
 	dim_c[5]; /* Dimming code: not supported by all terminals */
