@@ -8,7 +8,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -60,6 +60,7 @@ char
     ICON_ARCHIVE[] = NERD_ARCHIVE,
     ICON_AUDIO[] = NERD_MUSICFILE,
     ICON_BINARY[] = NERD_BINARY,
+    ICON_BOOK[] = NERD_BOOK,
     ICON_C[] = NERD_C,
     ICON_CD[] = NERD_OPTICALDISK,
 	ICON_CHESS[] = NERD_CHESS,
@@ -77,7 +78,6 @@ char
     ICON_DART[] = NERD_DART,
     ICON_DATABASE[] = NERD_DATABASE,
     ICON_DIFF[] = NERD_DIFF,
-    ICON_DJVU[] = NERD_DJVU,
     ICON_DOCKER[] = NERD_DOCKER,
     ICON_ELECTRON[] = NERD_ELECTRON,
     ICON_ELF[] = NERD_BINARY,
@@ -107,7 +107,9 @@ char
     ICON_MYSQL[] = NERD_MYSQL,
     ICON_ASM[] = NERD_ASM,
     ICON_OCAML[] = NERD_OCAML,
-    ICON_OPENOFFICE[] = NERD_FILE,
+    ICON_OPENOFFICE_P[] = NERD_PPTDOC,
+    ICON_OPENOFFICE_S[] = NERD_EXCELDOC,
+    ICON_OPENOFFICE_T[] = NERD_WORDDOC,
     ICON_PATCH[] = NERD_PATCH,
     ICON_PDF[] = NERD_PDF,
     ICON_PERL[] = NERD_PERL,
@@ -137,6 +139,7 @@ char
     ICON_VISUALSTUDIO[] = NERD_VISUALSTUDIO,
     ICON_WINDOWS[] = NERD_WINDOWS,
     ICON_WORD[] = NERD_WORDDOC,
+    ICON_ZIG[] = NERD_ZIG,
 
     ICON_ARCH[] = NERD_ARCHLINUX,
     ICON_DEBIAN[] = NERD_DEBIAN,
@@ -174,6 +177,7 @@ char
     ICON_ARCHIVE[] = FA_ARCHIVE,
     ICON_AUDIO[] = FA_FILE_AUDIO_O,
     ICON_BINARY[] = OCT_FILE_BINARY,
+    ICON_BOOK[] = OCT_BOOK,
     ICON_C[] = MFIZZ_C,
     ICON_CD[] = LINEA_MUSIC_CD,
     ICON_CHESS[] = FA_FILE_O,
@@ -191,7 +195,6 @@ char
     ICON_DART[] = DEV_DART,
     ICON_DATABASE[] = FA_DATABASE,
     ICON_DIFF[] = FILE_DIFF,
-    ICON_DJVU[] = FA_PAPERCLIP,
     ICON_DOCKER[] = MFIZZ_DOCKER,
     ICON_ELECTRON[] = FILE_ELECTRON,
     ICON_ELF[] = OCT_FILE_BINARY,
@@ -221,7 +224,9 @@ char
     ICON_MYSQL[] = MFIZZ_MYSQL,
     ICON_ASM[] = FILE_NASM,
     ICON_OCAML[] = FILE_OCAML,
-    ICON_OPENOFFICE[] = FILE_OPENOFFICE,
+    ICON_OPENOFFICE_P[] = FILE_OPENOFFICE,
+    ICON_OPENOFFICE_S[] = FILE_OPENOFFICE,
+    ICON_OPENOFFICE_T[] = FILE_OPENOFFICE,
     ICON_PATCH[] = FILE_PATCH,
     ICON_PDF[] = FA_FILE_PDF_O,
     ICON_PERL[] = MFIZZ_PERL,
@@ -251,6 +256,7 @@ char
     ICON_VISUALSTUDIO[] = DEV_VISUALSTUDIO,
     ICON_WINDOWS[] = DEV_WINDOWS,
     ICON_WORD[] = FILE_WORD,
+    ICON_ZIG[] = FA_FILE_O,
 
     ICON_ARCH[] = MFIZZ_ARCHLINUX,
     ICON_DEBIAN[] = MFIZZ_DEBIAN,
@@ -288,6 +294,7 @@ char
     ICON_ARCHIVE[] = EMOJI_ARCHIVE,
     ICON_AUDIO[] = EMOJI_AUDIO,
     ICON_BINARY[] = EMOJI_BINARY,
+    ICON_BOOK[] = EMOJI_BOOK_OPEN,
     ICON_C[] = EMOJI_C,
     ICON_CD[] = EMOJI_DISK,
     ICON_CHESS[] = EMOJI_CHESS,
@@ -305,7 +312,6 @@ char
     ICON_DART[] = EMOJI_DART,
     ICON_DATABASE[] = EMOJI_DATABASE,
     ICON_DIFF[] = EMOJI_DIFF,
-    ICON_DJVU[] = EMOJI_DJVU,
     ICON_DOCKER[] = EMOJI_DOCKER,
     ICON_ELECTRON[] = EMOJI_ELECTRON,
     ICON_ELF[] = EMOJI_BINARY,
@@ -335,7 +341,9 @@ char
     ICON_MYSQL[] = EMOJI_DATABASE,
     ICON_ASM[] = EMOJI_ASM,
     ICON_OCAML[] = EMOJI_OCAML,
-    ICON_OPENOFFICE[] = EMOJI_WORD,
+    ICON_OPENOFFICE_P[] = EMOJI_PRESENTATION,
+    ICON_OPENOFFICE_S[] = EMOJI_STYLESHEET,
+    ICON_OPENOFFICE_T[] = EMOJI_WORD,
     ICON_PATCH[] = EMOJI_PATCH,
     ICON_PDF[] = EMOJI_PDF,
     ICON_PERL[] = EMOJI_PERL,
@@ -365,6 +373,7 @@ char
     ICON_VISUALSTUDIO[] = EMOJI_VISUALSTUDIO,
     ICON_WINDOWS[] = EMOJI_WINDOWS,
     ICON_WORD[] = EMOJI_WORD,
+	ICON_ZIG[] = EMOJI_FILE,
 
     ICON_ARCH[] = EMOJI_MAKE,
     ICON_REDHAT[] = EMOJI_ARCHIVE,
@@ -434,8 +443,9 @@ char
 #define DEF_FILE_ICON_COLOR WHITE
 
 /* Per file extension icons */
-struct icons_t icon_ext[] = {
+struct icons_t const icon_ext[] = {
     {"7z", ICON_ARCHIVE, YELLOW},
+    {"3gp", ICON_VID, BLUE},
 
     {"a", ICON_ARCHIVE, YELLOW},
     {"apk", ICON_ARCHIVE, YELLOW},
@@ -443,6 +453,8 @@ struct icons_t icon_ext[] = {
     {"aup", ICON_AUDIO, YELLOW},
     {"avi", ICON_VID, BLUE},
     {"awk", ICON_SCRIPT, WHITE},
+    {"azw", ICON_BOOK, B_WHITE},
+    {"azw3", ICON_BOOK, B_WHITE},
 
     {"bash", ICON_SCRIPT, WHITE},
     {"bat", ICON_SCRIPT, WHITE},
@@ -457,6 +469,8 @@ struct icons_t icon_ext[] = {
     {"c++", ICON_CPP, WHITE},
     {"cabal", ICON_HASKELL, BLUE},
     {"cab", ICON_ARCHIVE, YELLOW},
+    {"cb7", ICON_ARCHIVE, YELLOW},
+    {"cba", ICON_ARCHIVE, YELLOW},
     {"cbr", ICON_ARCHIVE, YELLOW},
     {"cbz", ICON_ARCHIVE, YELLOW},
     {"cc", ICON_CPP, WHITE},
@@ -483,7 +497,7 @@ struct icons_t icon_ext[] = {
     {"db", ICON_DATABASE, WHITE},
     {"deb", ICON_DEBIAN, RED},
     {"diff", ICON_DIFF, WHITE},
-    {"djvu", ICON_DJVU, BLUE},
+    {"djvu", ICON_BOOK, B_WHITE},
     {"dll", ICON_SHARE, BLUE},
     {"doc", ICON_WORD, BLUE},
     {"docx", ICON_WORD, BLUE},
@@ -493,7 +507,7 @@ struct icons_t icon_ext[] = {
     {"el", ICON_EMACS, BLUE},
     {"elf", ICON_ELF, WHITE},
     {"elm", ICON_ELM, GREEN},
-    {"epub", ICON_PDF, B_RED},
+    {"epub", ICON_BOOK, B_WHITE},
     {"erb", ICON_RUBY, RED},
     {"erl", ICON_ERLANG, RED},
     {"ex", ICON_ELIXIR, BLUE},
@@ -505,6 +519,9 @@ struct icons_t icon_ext[] = {
     {"fish", ICON_SCRIPT, WHITE},
     {"flac", ICON_AUDIO, YELLOW},
     {"flv", ICON_VID, BLUE},
+    {"fodp", ICON_OPENOFFICE_P, YELLOW},
+    {"fods", ICON_OPENOFFICE_S, GREEN},
+    {"fodt", ICON_OPENOFFICE_T, BLUE},
     {"fs", ICON_FSHARP, WHITE},
     {"fsi", ICON_FSHARP, WHITE},
     {"fsscript", ICON_FSHARP, WHITE},
@@ -546,6 +563,8 @@ struct icons_t icon_ext[] = {
     {"jxl", ICON_IMG, GREEN},
 
     {"key", ICON_KEY, YELLOW},
+    {"kf8", ICON_BOOK, B_WHITE},
+    {"kfx", ICON_BOOK, B_WHITE},
     {"ko", ICON_BINARY, WHITE},
     {"kt", ICON_KOTLIN, BLUE},
     {"kts", ICON_KOTLIN, BLUE},
@@ -577,9 +596,9 @@ struct icons_t icon_ext[] = {
     {"nix", ICON_NIX, BLUE},
 
     {"o", ICON_BINARY, WHITE},
-    {"odp", ICON_OPENOFFICE, BLUE},
-    {"odt", ICON_OPENOFFICE, BLUE},
-    {"ods", ICON_OPENOFFICE, BLUE},
+    {"odp", ICON_OPENOFFICE_P, YELLOW},
+    {"ods", ICON_OPENOFFICE_S, GREEN},
+    {"odt", ICON_OPENOFFICE_T, BLUE},
     {"ogg", ICON_AUDIO, YELLOW},
     {"ogv", ICON_VID, BLUE},
     {"opdownload", ICON_DOWNLOADS, WHITE},
@@ -609,6 +628,9 @@ struct icons_t icon_ext[] = {
     {"pyd", ICON_PYTHON, GREEN},
     {"pyo", ICON_PYTHON, GREEN},
 
+    {"qcow", ICON_CD, B_WHITE},
+    {"qcow2", ICON_CD, B_WHITE},
+
     {"r", ICON_R, WHITE},
     {"rda", ICON_R, WHITE},
     {"rdata", ICON_R, WHITE},
@@ -650,6 +672,10 @@ struct icons_t icon_ext[] = {
     {"txt", ICON_TXT, WHITE},
     {"txz", ICON_ARCHIVE, YELLOW},
 
+    {"vdi", ICON_CD, B_WHITE},
+    {"vhd", ICON_CD, B_WHITE},
+    {"vhdx", ICON_CD, B_WHITE},
+    {"vmdk", ICON_CD, B_WHITE},
     {"vid", ICON_VID, BLUE},
     {"vim", ICON_VIM, GREEN},
     {"vimrc", ICON_VIM, GREEN},
@@ -673,13 +699,14 @@ struct icons_t icon_ext[] = {
     {"yaml", ICON_CONF, WHITE},
     {"yml", ICON_CONF, WHITE},
 
+    {"zig", ICON_ZIG, YELLOW},
     {"zip", ICON_ARCHIVE, YELLOW},
     {"zsh", ICON_SCRIPT, WHITE},
     {"zst", ICON_ARCHIVE, YELLOW},
 };
 
 /* Icons for some specific dir names */
-struct icons_t icon_dirnames[] = {
+struct icons_t const icon_dirnames[] = {
 /* Translated version for these dirs should be added here
  * See https://github.com/alexanderjeurissen/ranger_devicons/blob/main/devicons.py*/
     {"home", ICON_HOME, DEF_DIR_ICON_COLOR},
@@ -699,7 +726,7 @@ struct icons_t icon_dirnames[] = {
 };
 
 /* Icons for some specific file names */
-struct icons_t icon_filenames[] = {
+struct icons_t const icon_filenames[] = {
 /* More specific filenames from here https://github.com/alexanderjeurissen/ranger_devicons/blob/main/devicons.py */
     {".bash_history", ICON_CONF, WHITE},
     {".bash_logout", ICON_CONF, WHITE},
