@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -27,9 +27,9 @@
 
 __BEGIN_DECLS
 
-int  add_to_jumpdb(char *);
+int  add_to_jumpdb(char *dir);
 void save_jumpdb(void);
-int  dirjump(char **, int);
+int  dirjump(char **args, int mode);
 
 __END_DECLS
 
