@@ -163,6 +163,34 @@ kbinds_edit(char *app)
 	return FUNC_SUCCESS;
 }
 
+static int
+check_kbinds_conflict(void)
+{
+	if (kbinds_n == 0) {
+		puts(_("kb: No keybindings defined"));
+		return FUNC_SUCCESS;
+	}
+
+	int ret = FUNC_SUCCESS;
+	size_t i, j;
+	for (i = 0; i < kbinds_n; i++) {
+		for (j = i + 1; j < kbinds_n; j++) {
+			if (strcmp(kbinds[i].key, kbinds[j].key) == 0) {
+				fprintf(stderr, _("kb: '%s' conflicts with '%s'\n"),
+					kbinds[i].function, kbinds[j].function);
+				ret = FUNC_FAILURE;
+			}
+		}
+	}
+
+	if (ret == 0)
+		puts(_("kb: No conflict found"));
+	else
+		puts(_("kb: Run 'kb edit' to fix the conflict"));
+
+	return ret;
+}
+
 int
 kbinds_function(char **args)
 {
@@ -186,6 +214,9 @@ kbinds_function(char **args)
 		return FUNC_SUCCESS;
 	}
 
+	if (*args[1] == 'c' && strcmp(args[1], "conflict") == 0)
+		return check_kbinds_conflict();
+
 	if (*args[1] == 'e' && strcmp(args[1], "edit") == 0)
 		return kbinds_edit(args[2] ? args[2] : NULL);
 
@@ -1685,86 +1716,57 @@ rl_dir_pinned(int count, int key)
 	return run_kb_cmd(",");
 }
 
+/* Switch to workspace N. */
+static int
+rl_switch_workspace(const int n)
+{
+	if (rl_line_buffer && *rl_line_buffer)
+		rl_delete_text(0, rl_end);
+
+	char t[16];
+	exec_prompt_cmds = 1;
+
+	if (cur_ws == (n - 1)) {
+		/* If the user attempts to switch to the same workspace they're
+		 * currently in, switch rather to the previous workspace. */
+		if (prev_ws != cur_ws) {
+			snprintf(t, sizeof(t), "ws %d", prev_ws + 1);
+			return run_kb_cmd(t);
+		}
+
+		return FUNC_SUCCESS;
+	}
+
+	snprintf(t, sizeof(t), "ws %d", n);
+	return run_kb_cmd(t);
+}
+
 static int
 rl_ws1(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	if (rl_line_buffer && *rl_line_buffer)
-		rl_delete_text(0, rl_end);
-
-	exec_prompt_cmds = 1;
-	if (cur_ws == 0) {
-		/* If the user attempts to switch to the same workspace they're
-		 * currently in, switch rather to the previous workspace. */
-		if (prev_ws != cur_ws) {
-			char t[16];
-			snprintf(t, sizeof(t), "ws %d", prev_ws + 1);
-			return run_kb_cmd(t);
-		}
-		return FUNC_SUCCESS;
-	}
-
-	return run_kb_cmd("ws 1");
+	return rl_switch_workspace(1);
 }
 
 static int
 rl_ws2(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	if (rl_line_buffer && *rl_line_buffer)
-		rl_delete_text(0, rl_end);
-
-	exec_prompt_cmds = 1;
-	if (cur_ws == 1) {
-		if (prev_ws != cur_ws) {
-			char t[16];
-			snprintf(t, sizeof(t), "ws %d", prev_ws + 1);
-			return run_kb_cmd(t);
-		}
-		return FUNC_SUCCESS;
-	}
-
-	return run_kb_cmd("ws 2");
+	return rl_switch_workspace(2);
 }
 
 static int
 rl_ws3(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	if (rl_line_buffer && *rl_line_buffer)
-		rl_delete_text(0, rl_end);
-
-	exec_prompt_cmds = 1;
-	if (cur_ws == 2) {
-		if (prev_ws != cur_ws) {
-			char t[16];
-			snprintf(t, sizeof(t), "ws %d", prev_ws + 1);
-			return run_kb_cmd(t);
-		}
-		return FUNC_SUCCESS;
-	}
-
-	return run_kb_cmd("ws 3");
+	return rl_switch_workspace(3);
 }
 
 static int
 rl_ws4(int count, int key)
 {
 	UNUSED(count); UNUSED(key);
-	if (rl_line_buffer && *rl_line_buffer)
-		rl_delete_text(0, rl_end);
-
-	exec_prompt_cmds = 1;
-	if (cur_ws == 3) {
-		if (prev_ws != cur_ws) {
-			char t[16];
-			snprintf(t, sizeof(t), "ws %d", prev_ws + 1);
-			return run_kb_cmd(t);
-		}
-		return FUNC_SUCCESS;
-	}
-
-	return run_kb_cmd("ws 4");
+	return rl_switch_workspace(4);
 }
 
 static int
@@ -2161,6 +2163,29 @@ rl_run_pager(int count, int key)
 	return run_kb_cmd("pg");
 }
 
+/* Run "!<TAB>" to display the command history via finder. */
+static int
+rl_cmdhist_tab(int count, int key)
+{
+	UNUSED(count); UNUSED(key);
+	rl_insert_text("!");
+	rl_point = rl_end;
+
+#ifndef _NO_SUGGESTIONS
+	if (suggestion_buf)
+		clear_suggestion(CS_FREEBUF);
+#endif /* _NO_SUGGESTIONS */
+
+	tab_complete('!');
+
+	if (rl_end > 0 && rl_line_buffer[rl_end - 1] == '!') {
+		rl_end--;
+		rl_point--;
+	}
+
+	return FUNC_SUCCESS;
+}
+
 /* Used to disable keybindings. */
 static int
 do_nothing(int count, int key)
@@ -2263,6 +2288,7 @@ set_keybinds_from_file(void)
 	rl_bind_keyseq(find_key("prepend-sudo"), rl_prepend_sudo);
 	rl_bind_keyseq(find_key("toggle-disk-usage"), rl_toggle_disk_usage);
 	rl_bind_keyseq(find_key("toggle-max-name-len"), rl_toggle_max_filename_len);
+	rl_bind_keyseq(find_key("cmd-hist"), rl_cmdhist_tab);
 	rl_bind_keyseq(find_key("quit"), rl_quit);
 
 	/* Plugins */
