@@ -93,6 +93,7 @@
 #include "sanitize.h"
 #include "sort.h"
 #include "spawn.h"
+#include "xdu.h"        /* dir_size() */
 
 /* In case we want to try some faster printf implementation */
 /*#if defined(_PALAND_PRINTF)
@@ -970,7 +971,7 @@ compute_maxes(void)
 		}
 
 		if (prop_fields.links == 1) {
-			t = DIGINUM(file_info[i].linkn);
+			t = DIGINUM((unsigned int)file_info[i].linkn);
 			if (t > maxes.links)
 				maxes.links = t;
 		}
