@@ -44,13 +44,10 @@
 #define PERM_STR_LEN  ((MAX_COLOR * 14) + 16) /* construct_file_perms() */
 
 #define TIME_STR_LEN  (MAX_TIME_STR + (MAX_COLOR * 2) + 2) /* construct_timestamp() */
-
 /* construct_human_size() returns a string of at most MAX_HUMAN_SIZE chars (helpers.h) */
 #define SIZE_STR_LEN  (MAX_HUMAN_SIZE + (MAX_COLOR * 3) + 10) /* construct_file_size() */
-
 /* 2 colors + 2 names + (space + NUL byte) + DIM */
 #define ID_STR_LEN    ((MAX_COLOR * 2) + (NAME_MAX * 2) + 2 + 4)
-
 /* Max inode number able to hold: 999 billions! Padding could be as long
  * as max inode lenght - 1 */
 #define INO_STR_LEN   ((MAX_COLOR * 2) + ((12 + 1) * 2) + 4)
@@ -58,7 +55,6 @@
 #define LINKS_STR_LEN ((MAX_COLOR * 2) + 32)
 /* Files counter */
 #define FC_STR_LEN    ((MAX_COLOR * 2) + 32)
-
 /* File allocated blocks */
 #define BLK_STR_LEN   ((MAX_COLOR * 2) + 32)
 
@@ -149,7 +145,7 @@ construct_and_print_filename(const struct fileinfo *props,
 	int diff = 0;
 	char *ext_name = (char *)NULL;
 	if (cur_len > (size_t)max_namelen) {
-		int rest = (int)cur_len - max_namelen;
+		const int rest = (int)cur_len - max_namelen;
 		trim = TRIM_NO_EXT;
 		size_t ext_len = 0;
 		ext_name = get_ext_info_long(props->name, plen, &trim, &ext_len);
@@ -248,7 +244,7 @@ construct_file_size(const struct fileinfo *props, char *size_str,
 		&& props->du_status != 0);
 	const char *unit_color = conf.colorize == 0
 		? (du_err == 1 ? "\x1b[1m" : "")
-		: (du_err == 1 ? xf_c : dim_c);
+		: (du_err == 1 ? xf_cb : dim_c);
 
 	snprintf(size_str, SIZE_STR_LEN, "%s%*s%s%c\x1b[0m%s",
 		csize, size_max,
@@ -397,9 +393,9 @@ construct_files_counter(const struct fileinfo *props, char *fc_str,
 			(int)props->filesn, df_c);
 	} else {
 		snprintf(fc_str, FC_STR_LEN, "%s%*c%s", dn_c, max,
-		props->filesn < 0 ? '?' /* Dir with no read permission */
-		: (props->dir == 1 ? '0' : '-'),
-		df_c);
+			props->filesn < 0 ? '?' /* Dir with no read permission */
+			: (props->dir == 1 ? '0' : '-'),
+			df_c);
 	}
 }
 
@@ -545,7 +541,7 @@ print_entry_props(const struct fileinfo *props, const struct maxes_t *maxes,
 	*xattr_str = have_xattr == 1 ? (props->xattr == 1 ? XATTR_CHAR : ' ') : 0;
 
 	/* Print stuff */
-	for (size_t i = 0; prop_fields_str[i] && i < PROP_FIELDS_SIZE; i++) {
+	for (size_t i = 0; i < PROP_FIELDS_SIZE && prop_fields_str[i]; i++) {
 		int print_space = prop_fields_str[i + 1] ? 1 : 0;
 
 		switch (prop_fields_str[i]) {
