@@ -166,6 +166,7 @@ int
 	exit_code = 0,
 	follow_symlinks = UNSET,
 	fzftab = UNSET,
+	fzf_border_type = UNSET,
 	fzf_height_set = 0,
 	fzf_preview_border_type = 0,
 	hist_status = UNSET,
@@ -426,6 +427,7 @@ char
 	tx_c[MAX_COLOR + 2], /* Text color */
 	xs_c[MAX_COLOR + 2], /* Exit code: success */
 	xf_c[MAX_COLOR + 2], /* Exit code: failure */
+	xf_cb[MAX_COLOR],    /* Exit code: failure (dir read) */
 
 	tmp_color[MAX_COLOR + 2], /* A temp buffer to store color codes */
 	dim_c[] = "\x1b[2m";
@@ -698,6 +700,8 @@ const struct cmdslist_t param_str[] = {
 	{"icons off", 9},
 	{"j --purge", 9},
 	{"j --edit", 8},
+	{"kb conflict", 11},
+	{"keybinds conflict", 17},
 	{"kb edit", 7},
 	{"keybinds edit", 13},
 	{"kb list", 7},
