@@ -591,7 +591,7 @@ link points to (instead of for the link itself) is displayed."
 
 #define KB_USAGE "Manage key bindings\n\n\
 \x1b[1mUSAGE\x1b[0m\n\
-  kb, keybinds [list | edit [APP] | reset | readline]\n\n\
+  kb, keybinds [list | edit [APP] | conflict | reset | readline]\n\n\
 \x1b[1mEXAMPLES\x1b[0m\n\
 - List your current key bindings\n\
     kb (or 'kb list')\n\
@@ -599,6 +599,8 @@ link points to (instead of for the link itself) is displayed."
     kb edit\n\
 - Open/edit the key bindings file using mousepad\n\
     kb edit mousepad\n\
+- Detect key bindings conflicts\n\
+    kb conflict\n\
 - List the current key bindings for readline\n\
     kb readline\n\
 - Reset your key bindings settings\n\
@@ -1388,7 +1390,7 @@ and paths\
 \n      --max-files=NUM\t\t List only up to NUM files\
 \n      --max-path=NUM\t\t Abbreviate current directory in prompt after NUM characters \
 (if \\z is used in the prompt line)\
-\n      --mnt-udisks2\t\t Use 'udisks2' instead of 'udevil' for the 'media' command\
+\n      --mnt-udisks2\t\t Use udisks2(1) instead of udevil(1) for the 'media' command\
 \n      --no-apparent-size\t Inform file sizes as used blocks instead of used bytes (apparent size)\
 \n      --no-bold\t\t\t Disable bold colors (applies to all color schemes)\
 \n      --no-cd-auto\t\t Disable the autocd function\
@@ -1417,6 +1419,7 @@ and paths\
 \n      --opener=APPLICATION\t Resource opener to use instead of Lira, \
 CliFM's built-in opener\
 \n      --pager-view=MODE\t\t How to list files in the pager (auto, long, short)\
+\n      --physical-size\t\t Same as --no-apparent-size\
 \n      --preview=FILE\t\t Display a preview of FILE (via Shotgun) and exit\
 \n      --print-sel\t\t Keep the list of selected files in sight\n"
 
