@@ -2,7 +2,7 @@
  * 
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -38,14 +38,12 @@
 #include <errno.h>
 
 #include "aux.h"
+#include "cleaner_table.h"
 #include "file_operations.h"
 #include "history.h"
-#ifdef GENERIC_FS_MONITOR
-# include "listing.h" /* reload_dirlist() */
-#endif /* GENERIC_FS_MONITOR */
+#include "listing.h" /* reload_dirlist() */
 #include "messages.h"
 #include "misc.h"
-#include "cleaner_table.h"
 #include "readline.h"
 #include "selection.h"
 
@@ -344,92 +342,56 @@ edit_replacements(struct bleach_t *bfiles, size_t *n, int *edited_names)
 
 	*edited_names = 1;
 
-	char f[PATH_MAX];
+	char f[PATH_MAX + 1];
 	snprintf(f, sizeof(f), "%s/%s", (xargs.stealth_mode == 1)
 		? P_tmpdir : tmp_dir, TMP_FILENAME);
 
 	int fd = mkstemp(f);
-	if (fd == -1) {
-		err('e', PRINT_PROMPT, "bleach: mkstemp: %s: %s\n", f, strerror(errno));
-		return (struct bleach_t *)NULL;
-	}
+	if (fd == -1)
+		goto ERROR;
 
-	FILE *fp = (FILE *)NULL;
+	FILE *fp = fdopen(fd, "w");
+	if (!fp)
+		goto ERROR;
 
-#ifndef HAVE_DPRINTF
-	fp = fopen(f, "w");
-	if (!fp) {
-		if (unlink(f) == -1)
-			err('w', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
-		err('e', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
-		return (struct bleach_t *)NULL;
-	}
-#endif /* !HAVE_DPRINT */
-
-#ifdef HAVE_DPRINTF
-	dprintf(fd, BLEACH_TMP_HEADER);
-#else
 	fprintf(fp, BLEACH_TMP_HEADER);
-#endif /* HAVE_DPRINTF */
 
 	size_t i;
 	/* Copy all files to be renamed to the temp file */
 	for (i = 0; i < *n; i++) {
-#ifdef HAVE_DPRINTF
-		dprintf(fd, "original: %s\nreplacement: %s\n\n",
-			bfiles[i].original, bfiles[i].replacement);
-#else
 		fprintf(fp, "original: %s\nreplacement: %s\n\n",
 			bfiles[i].original, bfiles[i].replacement);
-#endif /* HAVE_DPRINTF */
 	}
 	size_t total_files = i;
 
-#if defined(__HAIKU__) || defined(__sun)
-	fclose(fp);
-#endif /* __HAIKU__ || __sun */
-	close(fd);
-
-	fp = open_fread(f, &fd);
-	if (!fp) {
-		if (unlink(f) == -1)
-			err('w', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
-		err('e', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
-		return (struct bleach_t *)NULL;
-	}
-
 	struct stat attr;
-	fstat(fd, &attr);
-	time_t mtime_bfr = (time_t)attr.st_mtime;
+	if (fstat(fd, &attr) == -1)
+		goto ERROR_CLOSE;
+	const time_t mtime_bfr = attr.st_mtime;
+
+	fclose(fp);
 
 	/* Open the temp file */
 	open_in_foreground = 1;
-	int exit_status = open_file(f);
+	const int exit_status = open_file(f);
 	open_in_foreground = 0;
-	if (exit_status != EXIT_SUCCESS) {
-		xerror("bleach: %s: %s\n", f, strerror(errno));
-		if (unlink(f) == -1)
-			xerror("bleach: %s: %s\n", f, strerror(errno));
-		fclose(fp);
-		return (struct bleach_t *)NULL;
-	}
+	if (exit_status != FUNC_SUCCESS)
+		goto ERROR;
 
-	fclose(fp);
 	fp = open_fread(f, &fd);
-	if (!fp) {
-		if (unlink(f) == -1)
-			err('w', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
-		err('e', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
-		return (struct bleach_t *)NULL;
-	}
+	if (!fp)
+		goto ERROR;
 
 	/* Compare the new modification time to the stored one: if they
 	 * match, nothing has been modified. */
-	fstat(fd, &attr);
-	if (mtime_bfr == (time_t)attr.st_mtime) {
-		if (unlinkat(fd, f, 0) == -1)
-			err('w', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
+	if (fstat(fd, &attr) == -1)
+		goto ERROR_CLOSE;
+
+	if (mtime_bfr == attr.st_mtime) {
 		fclose(fp);
+		if (unlinkat(fd, f, 0) == -1)
+			err('w', PRINT_PROMPT, "bleach: Cannot remove '%s': %s\n",
+				f, strerror(errno));
 		*edited_names = 0;
 		return bfiles; /* Return the original list of files */
 	}
@@ -443,7 +405,7 @@ edit_replacements(struct bleach_t *bfiles, size_t *n, int *edited_names)
 	/* Allocate memory for the new list */
 	free(bfiles);
 	bfiles = (struct bleach_t *)xnmalloc(total_files + 1,
-			sizeof(struct bleach_t));
+		sizeof(struct bleach_t));
 
 	/* Initialize all values */
 	for (i = 0; i < total_files; i++) {
@@ -507,10 +469,20 @@ edit_replacements(struct bleach_t *bfiles, size_t *n, int *edited_names)
 	free(line);
 
 	if (unlinkat(fd, f, 0) == -1)
-		err('w', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
+		err('w', PRINT_PROMPT, "bleach: Cannot remove '%s': %s\n",
+			f, strerror(errno));
 	fclose(fp);
 
 	return bfiles;
+
+ERROR_CLOSE:
+	fclose(fp);
+ERROR:
+	*edited_names = -1;
+	xerror("bleach: '%s': %s\n", f, strerror(errno));
+	if (unlink(f) == -1)
+		xerror("bleach: Cannot remove '%s': %s\n", f, strerror(errno));
+	return bfiles;
 }
 
 /* Clean up the list of file names (NAMES), print the list of the cleaned
@@ -521,16 +493,16 @@ bleach_files(char **names)
 {
 	if (!names || !names[1] || IS_HELP(names[1])) {
 		puts(_(BLEACH_USAGE));
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	struct bleach_t *bfiles = (struct bleach_t *)NULL;
 
 	size_t f = 0, i = 1;
 	for (; names[i]; i++) {
-		char *dstr = dequote_str(names[i], 0);
+		char *dstr = unescape_str(names[i], 0);
 		if (!dstr) {
-			xerror(_("bleach: %s: Error dequoting file name\n"), names[i]);
+			xerror(_("bleach: '%s': Error unescaping file name\n"), names[i]);
 			continue;
 		}
 
@@ -554,12 +526,12 @@ bleach_files(char **names)
 			continue;
 		}
 
-		bfiles = (struct bleach_t *)xrealloc(bfiles, (f + 1) * sizeof(struct bleach_t));
+		bfiles = xnrealloc(bfiles, f + 1, sizeof(struct bleach_t));
 		bfiles[f].original = savestring(names[i], nlen);
 		if (sl) {
 			*sl = '\0';
-			size_t len = nlen + strlen(p) + 2;
-			bfiles[f].replacement = (char *)xnmalloc(len, sizeof(char));
+			const size_t len = nlen + strlen(p) + 2;
+			bfiles[f].replacement = xnmalloc(len, sizeof(char));
 			snprintf(bfiles[f].replacement, len, "%s/%s", names[i], p);
 			*sl = '/';
 		} else {
@@ -573,7 +545,7 @@ bleach_files(char **names)
 
 	if (f == 0 || !bfiles) {
 		printf(_("%s: Nothing to do\n"), FUNC_NAME);
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	int rename = 0;
@@ -601,7 +573,7 @@ CONFIRM:
 		case 'e':
 			_edit = 1;
 			bfiles = edit_replacements(bfiles, &f, &edited_names);
-			if (!bfiles)
+			if (!bfiles || edited_names == -1)
 				break;
 
 			if (edited_names == 1 && f > 0) {
@@ -625,21 +597,30 @@ CONFIRM:
 		free(bfiles[0].replacement);
 		free(bfiles);
 		printf(_("%s: Nothing to do\n"), FUNC_NAME);
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
+	}
+
+	if (edited_names == -1) { /* ERROR */
+		for (i = 0; i < f; i++) {
+			free(bfiles[i].original);
+			free(bfiles[i].replacement);
+		}
+		free(bfiles);
+		return FUNC_FAILURE;
 	}
 
 	/* The user entered 'e' to edit the file, but nothing was modified
 	 * Ask for confirmation in case the user just wanted to see what would
 	 * be done. */
 	if (_edit == 1) {
-		printf(_("%zu %s will be bleached\n"), f, f > 1 ? _("files") : _("file"));
+		printf(_("%zu file name(s) will be bleached\n"), f);
 		if (rl_get_y_or_n(_("Continue? [y/n] ")) != 1) {
 			for (i = 0; i < f; i++) {
 				free(bfiles[i].original);
 				free(bfiles[i].replacement);
 			}
 			free(bfiles);
-			return EXIT_SUCCESS;
+			return FUNC_SUCCESS;
 		}
 	}
 
@@ -649,7 +630,7 @@ CONFIRM:
 
 	int total_rename = rename == 1 ? (int)f : 0;
 	size_t rep_suffix = 1;
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 
 	for (i = 0; i < f; i++) {
 		char *o = bfiles[i].original ? bfiles[i].original : (char *)NULL;
@@ -659,17 +640,18 @@ CONFIRM:
 			 * it does, append REP_SUFFIX and try again. */
 			struct stat a;
 			while (lstat(r, &a) == 0) {
-				char tmp[PATH_MAX];
+				char tmp[PATH_MAX + 1];
 				xstrsncpy(tmp, r, sizeof(tmp));
-				r = (char *)xrealloc(r, (PATH_MAX + 32) * sizeof(char));
+				r = xnrealloc(r, PATH_MAX + 32, sizeof(char));
 				snprintf(r, PATH_MAX + 31, "%s-%zu", tmp, rep_suffix);
 				rep_suffix++;
 			}
 
 			if (renameat(XAT_FDCWD, o, XAT_FDCWD, r) == -1) {
-				xerror("bleach: renameat: %s: %s\n", o, strerror(errno));
+				xerror("bleach: Cannot rename '%s' to '%s': %s\n",
+					o, r, strerror(errno));
 				total_rename--;
-				exit_status = EXIT_FAILURE;
+				exit_status = FUNC_FAILURE;
 			}
 		}
 
@@ -679,18 +661,12 @@ CONFIRM:
 
 	free(bfiles);
 
-	if (exit_status == EXIT_FAILURE || total_rename == 0) {
-		printf(_("%s: %d file(s) bleached\n"), FUNC_NAME, total_rename);
+	if (exit_status == FUNC_FAILURE || total_rename == 0) {
+		printf(_("%s: %d file names(s) bleached\n"), FUNC_NAME, total_rename);
 	} else {
-#ifdef GENERIC_FS_MONITOR
 		if (conf.autols == 1)
 			reload_dirlist();
-		print_reload_msg(_("%s: %d file(s) bleached\n"),
-			FUNC_NAME, total_rename);
-#else
-		err(ERR_NO_LOG, PRINT_PROMPT, _("%s->%s %s: %d file(s) bleached\n"),
-			mi_c, df_c, FUNC_NAME, total_rename);
-#endif /* GENERIC_FS_MONITOR */
+		print_reload_msg(_("%d file name(s) bleached\n"), total_rename);
 	}
 
 	return exit_status;
