@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -30,17 +30,16 @@
 #include <unistd.h>
 #include <readline/tilde.h>
 
-#include "aux.h"
-#include "checks.h"
-#include "colors.h" /* Used to get workspace path color */
-#include "fuzzy_match.h"
-#include "history.h"
-#include "jump.h"
-#include "listing.h"
-#include "misc.h"
+#include "aux.h"         /* mem.h, normalize_path, get_cwd */
+#include "checks.h"      /* is_number */
+#include "fuzzy_match.h" /* fuzzy_match, contains_utf8 */
+#include "history.h"     /* add_to_dirhist */
+#include "jump.h"        /* add_to_jumpdb */
+#include "listing.h"     /* reload_dirlist */
+#include "messages.h"    /* description and usage messages */
+#include "misc.h"        /* xerror */
 #include "navigation.h"
-#include "messages.h"
-#include "readline.h"
+#include "readline.h"    /* rl_no_hist */
 
 #define BD_CONTINUE 2
 
@@ -58,404 +57,40 @@ pwd_function(const char *arg)
 			resolve_links = 1;
 		} else if (IS_HELP(arg)) {
 			puts(PWD_DESC);
-			return EXIT_SUCCESS;
+			return FUNC_SUCCESS;
 		} else if (arg[1] != 'L') {
-			xerror("pwd: %s: Invalid option\nUsage: pwd [-LP]\n", arg);
-			return EXIT_FAILURE;
+			xerror(_("pwd: '%s': Invalid option\nUsage: pwd [-LP]\n"), arg);
+			return FUNC_FAILURE;
 		}
 	}
 
 	if (workspaces && workspaces[cur_ws].path) {
 		pwd = workspaces[cur_ws].path;
 	} else {
-		char p[PATH_MAX]; *p = '\0';
+		char p[PATH_MAX + 1]; *p = '\0';
 		pwd = get_cwd(p, sizeof(p), 0);
 	}
 
 	if (!pwd || !*pwd) {
 		xerror(_("%s: Error getting the current working directory\n"),
 			PROGRAM_NAME);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	if (resolve_links == 0) {
 		puts(pwd);
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
-	char p[PATH_MAX]; *p = '\0';
+	char p[PATH_MAX + 1]; *p = '\0';
 	char *real_path = realpath(pwd, p);
 	if (!real_path) {
-		xerror("pwd: %s: %s\n", pwd, strerror(errno));
+		xerror("pwd: '%s': %s\n", pwd, strerror(errno));
 		return errno;
 	}
 
 	puts(p);
-	return EXIT_SUCCESS;
-}
-
-static size_t
-get_longest_workspace_name(void)
-{
-	if (!workspaces)
-		return 0;
-
-	size_t longest_ws = 0;
-	int i = MAX_WS;
-
-	while (--i >= 0) {
-		if (!workspaces[i].name)
-			continue;
-		size_t l = wc_xstrlen(workspaces[i].name);
-		if (l > longest_ws)
-			longest_ws = l;
-	}
-
-	return longest_ws;
-}
-
-/*
-static char *
-get_workspace_color(const uint8_t num)
-{
-	if (conf.colorize == 0)
-		return df_c;
-
-	switch (num + 1) {
-	case 1: return *ws1_c ? ws1_c : DEF_WS1_C;
-	case 2: return *ws2_c ? ws2_c : DEF_WS2_C;
-	case 3: return *ws3_c ? ws3_c : DEF_WS3_C;
-	case 4: return *ws4_c ? ws4_c : DEF_WS4_C;
-	case 5: return *ws5_c ? ws5_c : DEF_WS5_C;
-	case 6: return *ws6_c ? ws6_c : DEF_WS6_C;
-	case 7: return *ws7_c ? ws7_c : DEF_WS7_C;
-	case 8: return *ws8_c ? ws8_c : DEF_WS8_C;
-	default: return df_c;
-	}
-} */
-
-static char *
-get_workspace_path_color(const uint8_t num)
-{
-	if (conf.colorize == 0)
-		return df_c;
-
-	if (!workspaces[num].path)
-		return DEF_DL_C; /* Unset. DL (dividing line) defaults to gray: let's use this */
-
-	struct stat a;
-	if (lstat(workspaces[num].path, &a) == -1)
-		return uf_c;
-
-	if (check_file_access(a.st_mode, a.st_uid, a.st_gid) == 0)
-		return nd_c;
-
-	if (S_ISLNK(a.st_mode)) {
-		char p[PATH_MAX];
-		*p = '\0';
-		char *ret = realpath(workspaces[num].path, p);
-		return (ret && *p) ? ln_c : or_c;
-	}
-
-	return get_dir_color(workspaces[num].path, a.st_mode, a.st_nlink, -1);
-}
-
-static int
-list_workspaces(void)
-{
-	uint8_t i;
-	int pad = (int)get_longest_workspace_name();
-
-	for (i = 0; i < MAX_WS; i++) {
-		char *path_color = get_workspace_path_color(i);
-
-		if (i == cur_ws)
-			printf("%s>%s ", mi_c, df_c);
-		else
-			fputs("  ", stdout);
-
-//		char *p = get_workspace_color(i);
-//		char *ws_color = p ? p : df_c;
-		char *ws_color = df_c;
-
-		if (workspaces[i].name) {
-			printf("%s%d%s [%s%s%s]: %*s",
-				ws_color, i + 1, df_c,
-				ws_color, workspaces[i].name, df_c,
-				pad - (int)wc_xstrlen(workspaces[i].name), "");
-		} else {
-			printf("%s%d%s: %*s",
-				ws_color, i + 1, df_c,
-				pad > 0 ? pad + 3 : 0, "");
-		}
-
-		printf("%s%s%s\n",
-			path_color,
-			workspaces[i].path ? workspaces[i].path : "unset",
-			df_c);
-	}
-
-	return EXIT_SUCCESS;
-}
-
-static int
-check_workspace_num(char *str, int *tmp_ws)
-{
-	int istr = atoi(str);
-	if (istr <= 0 || istr > MAX_WS) {
-		xerror(_("ws: %d: No such workspace (valid workspaces: "
-			"1-%d)\n"), istr, MAX_WS);
-		return EXIT_FAILURE;
-	}
-
-	*tmp_ws = istr - 1;
-
-	if (*tmp_ws == cur_ws) {
-		xerror(_("ws: %d: Is the current workspace\n"), *tmp_ws + 1);
-		return EXIT_SUCCESS;
-	}
-
-	return 2;
-}
-
-static void
-save_workspace_opts(const int n)
-{
-	free(workspace_opts[n].filter.str);
-	workspace_opts[n].filter.str = filter.str
-		? savestring(filter.str, strlen(filter.str)) : (char *)NULL;
-	workspace_opts[n].filter.rev = filter.rev;
-	workspace_opts[n].filter.type = filter.type;
-	workspace_opts[n].filter.env = filter.env;
-
-	workspace_opts[n].color_scheme = cur_cscheme;
-	workspace_opts[n].files_counter = conf.files_counter;
-	workspace_opts[n].light_mode = conf.light_mode;
-	workspace_opts[n].list_dirs_first = conf.list_dirs_first;
-	workspace_opts[n].long_view = conf.long_view;
-	workspace_opts[n].max_files = max_files;
-	workspace_opts[n].max_name_len = conf.max_name_len;
-	workspace_opts[n].only_dirs = conf.only_dirs;
-	workspace_opts[n].pager = conf.pager;
-	workspace_opts[n].show_hidden = conf.show_hidden;
-	workspace_opts[n].sort = conf.sort;
-	workspace_opts[n].sort_reverse = conf.sort_reverse;
-}
-
-static void
-unset_ws_filter(void)
-{
-	free(filter.str);
-	filter.str = (char *)NULL;
-	filter.rev = 0;
-	filter.type = FILTER_NONE;
-	regfree(&regex_exp);
-}
-
-static void
-set_ws_filter(const int n)
-{
-	filter.type = workspace_opts[n].filter.type;
-	filter.rev = workspace_opts[n].filter.rev;
-	filter.env = workspace_opts[n].filter.env;
-
-	free(filter.str);
-	regfree(&regex_exp);
-	char *p = workspace_opts[n].filter.str;
-	filter.str = savestring(p, strlen(p));
-
-	if (filter.type != FILTER_FILE_NAME)
-		return;
-
-	if (regcomp(&regex_exp, filter.str, REG_NOSUB | REG_EXTENDED) != EXIT_SUCCESS)
-		unset_ws_filter();
-}
-
-static void
-set_workspace_opts(const int n)
-{
-	if (workspace_opts[n].color_scheme
-	&& workspace_opts[n].color_scheme != cur_cscheme)
-		set_colors(workspace_opts[n].color_scheme, 0);
-
-	if (workspace_opts[n].filter.str && *workspace_opts[n].filter.str) {
-		set_ws_filter(n);
-	} else {
-		if (filter.str)
-			unset_ws_filter();
-	}
-
-	conf.light_mode = workspace_opts[n].light_mode;
-	conf.list_dirs_first = workspace_opts[n].list_dirs_first;
-	conf.long_view = workspace_opts[n].long_view;
-	conf.files_counter = workspace_opts[n].files_counter;
-	max_files = workspace_opts[n].max_files;
-	conf.max_name_len = workspace_opts[n].max_name_len;
-	conf.only_dirs = workspace_opts[n].only_dirs;
-	conf.pager = workspace_opts[n].pager;
-	conf.show_hidden = workspace_opts[n].show_hidden;
-	conf.sort = workspace_opts[n].sort;
-	conf.sort_reverse = workspace_opts[n].sort_reverse;
-}
-
-static int
-switch_workspace(const int tmp_ws)
-{
-	/* If new workspace has no path yet, copy the path of the current workspace */
-	if (!workspaces[tmp_ws].path) {
-		workspaces[tmp_ws].path = savestring(workspaces[cur_ws].path,
-		    strlen(workspaces[cur_ws].path));
-	} else {
-		if (access(workspaces[tmp_ws].path, R_OK | X_OK) != EXIT_SUCCESS) {
-			xerror("ws: %s: %s\n", workspaces[tmp_ws].path, strerror(errno));
-			free(workspaces[tmp_ws].path);
-			workspaces[tmp_ws].path = savestring(workspaces[cur_ws].path,
-				strlen(workspaces[cur_ws].path));
-		}
-	}
-
-	if (xchdir(workspaces[tmp_ws].path, SET_TITLE) == -1) {
-		xerror("ws: %s: %s\n", workspaces[tmp_ws].path, strerror(errno));
-		return EXIT_FAILURE;
-	}
-
-	if (conf.private_ws_settings == 1)
-		save_workspace_opts(cur_ws);
-
-	prev_ws = cur_ws;
-	cur_ws = tmp_ws;
-	dir_changed = 1;
-
-	if (conf.colorize == 1 && xargs.eln_use_workspace_color == 1)
-		set_eln_color();
-
-	if (conf.private_ws_settings == 1)
-		set_workspace_opts(cur_ws);
-
-	if (conf.autols == 1)
-		reload_dirlist();
-
-	add_to_dirhist(workspaces[cur_ws].path);
-	return EXIT_SUCCESS;
-}
-
-/* Return the workspace number corresponding to the workspace name NAME,
- * or -1 if no workspace is named NAME, if error, or if NAME is already
- * the current workspace */
-static int
-get_workspace_by_name(char *name, const int check_current)
-{
-	if (!workspaces || !name || !*name)
-		return (-1);
-
-	/* CHECK_CURRENT is zero when coming from unset_workspace(), in which
-	 * case name is already dequoted */
-	char *p = check_current == 1 ? dequote_str(name, 0) : (char *)NULL;
-	char *q = p ? p : name;
-
-	int n = MAX_WS;
-	while (--n >= 0) {
-		if (!workspaces[n].name || *workspaces[n].name != *q
-		|| strcmp(workspaces[n].name, q) != 0)
-			continue;
-		if (n == cur_ws && check_current == 1) {
-			xerror(_("ws: %s: Is the current workspace\n"), q);
-			free(p);
-			return (-1);
-		}
-		free(p);
-		return n;
-	}
-
-	xerror(_("ws: %s: No such workspace\n"), q);
-	free(p);
-	return (-1);
-}
-
-static int
-unset_workspace(char *str)
-{
-	int n = -1;
-
-	char *name = dequote_str(str, 0);
-	if (!name) {
-		xerror("ws: %s: Error dequoting name\n", str);
-		return EXIT_FAILURE;
-	}
-
-	if (!is_number(name)) {
-		if ((n = get_workspace_by_name(name, 0)) == -1)
-			goto ERROR;
-		n++;
-	} else {
-		n = atoi(name);
-	}
-
-	if (n < 1 || n > MAX_WS) {
-		xerror(_("ws: %s: No such workspace (valid workspaces: "
-			"1-%d)\n"), name, MAX_WS);
-		goto ERROR;
-	}
-
-	n--;
-	if (n == cur_ws) {
-		xerror(_("ws: %s: Is the current workspace\n"), name);
-		goto ERROR;
-	}
-
-	if (!workspaces[n].path) {
-		xerror(_("ws: %s: Already unset\n"), name);
-		goto ERROR;
-	}
-
-	printf(_("ws: %s: Workspace unset\n"), name);
-	free(name);
-
-	free(workspaces[n].path);
-	workspaces[n].path = (char *)NULL;
-
-	return EXIT_SUCCESS;
-
-ERROR:
-	free(name);
-	return EXIT_FAILURE;
-}
-
-int
-handle_workspaces(char **args)
-{
-	if (!args[0] || !*args[0])
-		return list_workspaces();
-
-	if (IS_HELP(args[0])) {
-		puts(_(WS_USAGE));
-		return EXIT_SUCCESS;
-	}
-
-	if (args[1] && strcmp(args[1], "unset") == 0)
-		return unset_workspace(args[0]);
-
-	int tmp_ws = 0;
-
-	if (is_number(args[0])) {
-		int ret = check_workspace_num(args[0], &tmp_ws);
-		if (ret != 2)
-			return ret;
-	} else if (*args[0] == '+' && !args[0][1]) {
-		if ((cur_ws + 1) >= MAX_WS)
-			return EXIT_FAILURE;
-		tmp_ws = cur_ws + 1;
-	} else if (*args[0] == '-' && !args[0][1]) {
-			if ((cur_ws - 1) < 0)
-				return EXIT_FAILURE;
-			tmp_ws = cur_ws - 1;
-	} else {
-		tmp_ws = get_workspace_by_name(args[0], 1);
-		if (tmp_ws == -1)
-			return EXIT_FAILURE;
-	}
-
-	return switch_workspace(tmp_ws);
+	return FUNC_SUCCESS;
 }
 
 /* Return the list of paths in CWD matching STR */
@@ -473,10 +108,10 @@ get_bd_matches(const char *str, int *n, const int mode)
 		 * that we need to reserve the first slot to hold the query
 		 * string */
 		*n = 1;
-		matches = (char **)xnmalloc(2, sizeof(char *));
+		matches = xnmalloc(2, sizeof(char *));
 	}
 
-	while(1) {
+	while (1) {
 		char *p = (char *)NULL;
 		if (str && *str) { /* Non-empty query string */
 			p = conf.case_sens_path_comp
@@ -484,6 +119,7 @@ get_bd_matches(const char *str, int *n, const int mode)
 			if (!p)
 				break;
 		}
+
 		char *q = strchr(p ? p : cwd, '/');
 		if (!q) {
 			if (!*(++cwd))
@@ -491,7 +127,8 @@ get_bd_matches(const char *str, int *n, const int mode)
 			continue;
 		}
 		*q = '\0';
-		matches = (char **)xrealloc(matches, (size_t)(*n + 2) * sizeof(char *));
+
+		matches = xnrealloc(matches, (size_t)(*n + 2), sizeof(char *));
 		if (mode == BD_TAB) {
 			/* Print only the path base name */
 			char *ss = strrchr(workspaces[cur_ws].path, '/');
@@ -509,6 +146,7 @@ get_bd_matches(const char *str, int *n, const int mode)
 			}
 			(*n)++;
 		}
+
 		*q = '/';
 		cwd = q + 1;
 
@@ -547,17 +185,17 @@ grab_bd_input(const int n)
 {
 	char *input = (char *)NULL;
 	putchar('\n');
+
 	while (!input) {
-		input = rl_no_hist("Choose a directory ('q' to quit): ");
-		if (!input)
-			continue;
-		if (!*input) {
+		input = rl_no_hist(_("Choose a directory ('q' to quit): "));
+		if (!input || !*input) {
 			free(input);
 			input = (char *)NULL;
 			continue;
 		}
+
 		if (is_number(input)) {
-			int a = atoi(input);
+			const int a = atoi(input);
 			if (a > 0 && a <= n) {
 				free(input);
 				return a - 1;
@@ -566,9 +204,11 @@ grab_bd_input(const int n)
 				input = (char *)NULL;
 				continue;
 			}
+
 		} else if (*input == 'q' && !*(input + 1)) {
 			free(input);
 			return (-1);
+
 		} else {
 			free(input);
 			input = (char *)NULL;
@@ -583,15 +223,15 @@ static int
 backdir_directory(char *dir, const char *str)
 {
 	if (!dir) {
-		xerror(_("bd: %s: Error dequoting string\n"), str);
-		return EXIT_FAILURE;
+		xerror(_("bd: '%s': Error unescaping string\n"), str);
+		return FUNC_FAILURE;
 	}
 
 	if (*dir == '~') {
 		char *exp_path = tilde_expand(dir);
 		if (!exp_path) {
-			xerror(_("bd: %s: Error expanding tilde\n"), dir);
-			return EXIT_FAILURE;
+			xerror(_("bd: '%s': Error expanding tilde\n"), dir);
+			return FUNC_FAILURE;
 		}
 		dir = exp_path;
 	}
@@ -624,11 +264,11 @@ backdir_menu(char **matches)
 		}
 	}
 
-	int choice = grab_bd_input(i);
+	const int choice = grab_bd_input(i);
 	if (choice != -1)
 		return cd_function(matches[choice], CD_PRINT_ERROR);
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
@@ -636,27 +276,27 @@ help_or_root(const char *str)
 {
 	if (str && IS_HELP(str)) {
 		puts(_(BD_USAGE));
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	if (*workspaces[cur_ws].path == '/' && !workspaces[cur_ws].path[1]) {
-		puts(_("bd: /: No parent directory"));
-		return EXIT_SUCCESS;
+		puts(_("bd: '/': No parent directory"));
+		return FUNC_SUCCESS;
 	}
 
-	return EXIT_FAILURE;
+	return FUNC_FAILURE;
 }
 
 /* Change to parent directory matching STR */
 int
-backdir(char* str)
+backdir(char *str)
 {
-	if (help_or_root(str) == EXIT_SUCCESS)
-		return EXIT_SUCCESS;
+	if (help_or_root(str) == FUNC_SUCCESS)
+		return FUNC_SUCCESS;
 
-	char *deq_str = str ? dequote_str(str, 0) : (char *)NULL;
+	char *deq_str = str ? unescape_str(str, 0) : (char *)NULL;
 	if (str) {
-		int ret = backdir_directory(deq_str, str);
+		const int ret = backdir_directory(deq_str, str);
 		if (ret != BD_CONTINUE) {
 			free(deq_str);
 			return ret;
@@ -665,7 +305,7 @@ backdir(char* str)
 
 	if (!workspaces[cur_ws].path) {
 		free(deq_str);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	int n = 0;
@@ -674,10 +314,10 @@ backdir(char* str)
 
 	if (n == 0) {
 		xerror(_("bd: %s: No matches found\n"), str);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
-	int exit_status = EXIT_SUCCESS, i = n;
+	int exit_status = FUNC_SUCCESS, i = n;
 	if (n == 1) /* Just one match: change to it */
 		exit_status = cd_function(matches[0], CD_PRINT_ERROR);
 	else if (n > 1) /* Multiple matches: print a menu to choose from */
@@ -686,6 +326,7 @@ backdir(char* str)
 	while (--i >= 0)
 		free(matches[i]);
 	free(matches);
+
 	return exit_status;
 }
 
@@ -716,10 +357,10 @@ xchdir(char *dir, const int cd_flag)
 
 	closedir(dirp);
 
-	int ret = chdir(dir);
+	const int ret = chdir(dir);
 
 	if (ret == 0 && cd_flag == SET_TITLE) {
-		char tmp[PATH_MAX]; *tmp = '\0';
+		char tmp[PATH_MAX + 1]; *tmp = '\0';
 		char *p = get_cwd(tmp, sizeof(tmp), 0);
 
 		/* Do not set OLDPWD if changing to the same directory ("cd ."
@@ -747,15 +388,19 @@ check_cdpath(const char *name)
 		return (char *)NULL;
 
 	size_t i;
-	char tmp[PATH_MAX];
+	const size_t namelen = strlen(name);
 	char *p = (char *)NULL;
 	struct stat a;
+
 	for (i = 0; cdpaths[i]; i++) {
-		size_t len = strlen(cdpaths[i]);
+		const size_t len = strlen(cdpaths[i]);
+		const size_t tmp_len = len + namelen + 2;
+		char *tmp = xnmalloc(tmp_len, sizeof(char));
+
 		if (len > 0 && cdpaths[i][len - 1] == '/')
-			snprintf(tmp, sizeof(tmp), "%s%s", cdpaths[i], name);
+			snprintf(tmp, tmp_len, "%s%s", cdpaths[i], name);
 		else
-			snprintf(tmp, sizeof(tmp), "%s/%s", cdpaths[i], name);
+			snprintf(tmp, tmp_len, "%s/%s", cdpaths[i], name);
 
 		char *exp_path = (char *)NULL;
 		if (*tmp == '~')
@@ -764,12 +409,20 @@ check_cdpath(const char *name)
 		char *dir = exp_path ? exp_path : tmp;
 		if (stat(dir, &a) != -1 && S_ISDIR(a.st_mode)) {
 			p = savestring(dir, strlen(dir));
+			free(tmp);
 			free(exp_path);
 			break;
 		}
+
+		free(tmp);
 		free(exp_path);
 	}
 
+	/* Print message (post_listing(), in listing.c) to let the user
+	 * know they changed to a dir in CDPATH. */
+	if (p)
+		is_cdpath = 1;
+
 	return p;
 }
 
@@ -783,11 +436,10 @@ go_home(const int cd_flag)
 		return ENOENT;
 	}
 
-	if (xchdir(user.home, SET_TITLE) != EXIT_SUCCESS) {
-		int tmp_err = errno;
+	if (xchdir(user.home, SET_TITLE) != FUNC_SUCCESS) {
 		if (cd_flag == CD_PRINT_ERROR)
-			xerror("cd: %s: %s\n", user.home, strerror(tmp_err));
-		return tmp_err;
+			xerror("cd: '%s': %s\n", user.home, strerror(errno));
+		return errno;
 	}
 
 	if (workspaces) {
@@ -795,7 +447,7 @@ go_home(const int cd_flag)
 		workspaces[cur_ws].path = savestring(user.home, strlen(user.home));
 	}
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Change current directory to NEW_PATH */
@@ -804,11 +456,11 @@ change_to_path(char *new_path, const int cd_flag)
 {
 	if (!new_path || !*new_path) {
 		xerror("%s\n", _("cd: Path is NULL or empty"));
-		return ENOENT;
+		return EINVAL;
 	}
 
 	if (strchr(new_path, '\\')) {
-		char *deq_path = dequote_str(new_path, 0);
+		char *deq_path = unescape_str(new_path, 0);
 		if (deq_path) {
 			xstrsncpy(new_path, deq_path, strlen(deq_path) + 1);
 			free(deq_path);
@@ -822,16 +474,16 @@ change_to_path(char *new_path, const int cd_flag)
 
 	if (!q) {
 		if (cd_flag == CD_PRINT_ERROR)
-			xerror(_("cd: %s: Error normalizing path\n"), new_path);
+			xerror(_("cd: '%s': Error normalizing path\n"), new_path);
 		free(p);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
+
 	free(p);
 
-	if (xchdir(q, SET_TITLE) != EXIT_SUCCESS) {
-		int tmp_err = errno;
+	if (xchdir(q, SET_TITLE) != FUNC_SUCCESS) {
 		if (cd_flag == CD_PRINT_ERROR)
-			xerror("cd: %s: %s\n", new_path, strerror(tmp_err));
+			xerror("cd: '%s': %s\n", new_path, strerror(errno));
 
 		free(q);
 
@@ -839,10 +491,10 @@ change_to_path(char *new_path, const int cd_flag)
 		 * a general error code, is not quite informative. Why not to return the
 		 * actual error code returned by chdir(3)? Note that POSIX only requires
 		 * for cd to return >0 in case of error (see cd(1p)). */
-		if (tmp_err == EACCES || tmp_err == ENOENT)
-			return EXIT_FAILURE;
+		if (errno == EACCES || errno == ENOENT)
+			return FUNC_FAILURE;
 
-		return tmp_err;
+		return errno;
 	}
 
 	if (workspaces) {
@@ -852,14 +504,14 @@ change_to_path(char *new_path, const int cd_flag)
 
 	free(q);
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
 skip_directory(const char *dir)
 {
 	return (conf.dirhistignore_regex && *conf.dirhistignore_regex
-		&& regexec(&regex_dirhist, dir, 0, NULL, 0) == EXIT_SUCCESS);
+		&& regexec(&regex_dirhist, dir, 0, NULL, 0) == FUNC_SUCCESS);
 }
 
 /* Change current directory to NEW_PATH, or to HOME if new_path is NULL.
@@ -868,10 +520,12 @@ int
 cd_function(char *new_path, const int cd_flag)
 {
 	/* If no argument, change to home */
-	int ret = EXIT_SUCCESS;
+	int ret = FUNC_SUCCESS;
+
 	if (!new_path || !*new_path) {
-		if ((ret = go_home(cd_flag)) != EXIT_SUCCESS)
+		if ((ret = go_home(cd_flag)) != FUNC_SUCCESS)
 			return ret;
+
 	} else if (*new_path == '-' && !new_path[1]) {
 		/* Implementation of the shell 'cd -' command */
 		static int state = 0;
@@ -883,8 +537,9 @@ cd_function(char *new_path, const int cd_flag)
 			state = 0;
 			return forth_function(c);
 		}
+
 	} else {
-		if ((ret = change_to_path(new_path, cd_flag)) != EXIT_SUCCESS)
+		if ((ret = change_to_path(new_path, cd_flag)) != FUNC_SUCCESS)
 			return ret;
 	}
 
@@ -894,11 +549,8 @@ cd_function(char *new_path, const int cd_flag)
 		add_to_dirhist(workspaces[cur_ws].path);
 
 	dir_changed = 1;
-	if (conf.autols == 1) {
-		free_dirlist();
-		if (list_dir() != EXIT_SUCCESS)
-			ret = EXIT_FAILURE;
-	}
+	if (conf.autols == 1)
+		reload_dirlist();
 
 	if (skip == 0)
 		add_to_jumpdb(workspaces[cur_ws].path);
@@ -944,8 +596,11 @@ fastback(char *str)
 		p++;
 	}
 
-	if (dots <= 2)
+	if (dots <= 2) {
+		if (dots == 2)
+			return normalize_path("..", 2);
 		return (char *)NULL;
+	}
 
 	char q[PATH_MAX];
 	q[0] = '.';
@@ -968,7 +623,7 @@ fastback(char *str)
 			i++;
 		}
 
-		xstrncat(q, strlen(q), rem, sizeof(q));
+		xstrncat(q, strnlen(q, sizeof(q)), rem, sizeof(q));
 	}
 
 	return normalize_path(q, i);
@@ -978,7 +633,7 @@ void
 print_dirhist(char *query)
 {
 	int n = DIGINUM(dirhist_total_index), i;
-	size_t len = (query && conf.fuzzy_match == 1) ? strlen(query) : 0;
+	const size_t len = (query && conf.fuzzy_match == 1) ? strlen(query) : 0;
 	int fuzzy_str_type = (len > 0 && contains_utf8(query) == 1)
 		? FUZZY_FILES_UTF8 : FUZZY_FILES_ASCII;
 
@@ -1010,9 +665,9 @@ clear_dirhist(void)
 	dirhist_cur_index = dirhist_total_index = 0;
 	add_to_dirhist(workspaces[cur_ws].path);
 
-	printf("%s: Directory history cleared\n", PROGRAM_NAME);
+	printf(_("%s: Directory history cleared\n"), PROGRAM_NAME);
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Change to the specified directory number (N) in the directory
@@ -1022,31 +677,29 @@ change_to_dirhist_num(int n)
 {
 	if (n <= 0 || n > dirhist_total_index) {
 		xerror(_("history: %d: No such ELN\n"), n);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	n--;
 	if (!old_pwd[n] || *old_pwd[n] == KEY_ESC) {
 		xerror("%s\n", _("history: Invalid history entry"));
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	int ret = xchdir(old_pwd[n], SET_TITLE);
 	if (ret != 0) {
-		xerror("history: %s: %s\n", old_pwd[n], strerror(errno));
-		return EXIT_FAILURE;
+		xerror("history: '%s': %s\n", old_pwd[n], strerror(errno));
+		return FUNC_FAILURE;
 	}
 
 	free(workspaces[cur_ws].path);
 	workspaces[cur_ws].path = savestring(old_pwd[n], strlen(old_pwd[n]));
 
 	dirhist_cur_index = n;
-	ret = EXIT_SUCCESS;
+	ret = FUNC_SUCCESS;
 
-	if (conf.autols == 1) {
-		free_dirlist();
-		ret = list_dir();
-	}
+	if (conf.autols == 1)
+		reload_dirlist();
 
 	return ret;
 }
@@ -1056,7 +709,7 @@ surf_hist(char **args)
 {
 	if (*args[1] == 'h' && (!args[1][1] || strcmp(args[1], "hist") == 0)) {
 		print_dirhist(NULL);
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	if (*args[1] == 'c' && strcmp(args[1], "clear") == 0)
@@ -1064,10 +717,10 @@ surf_hist(char **args)
 
 	if (*args[1] != '!' || is_number(args[1] + 1) != 1) {
 		fprintf(stderr, "%s\n", _(DIRHIST_USAGE));
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
-	int n = atoi(args[1] + 1);
+	const int n = atoi(args[1] + 1);
 	return change_to_dirhist_num(n);
 }
 
@@ -1078,18 +731,15 @@ set_path(const char *new_path)
 	free(workspaces[cur_ws].path);
 	workspaces[cur_ws].path = savestring(new_path, strlen(new_path));
 	if (!workspaces[cur_ws].path)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	add_to_jumpdb(workspaces[cur_ws].path);
-	int exit_status = EXIT_SUCCESS;
 
 	dir_changed = 1;
-	if (conf.autols == 1) {
-		free_dirlist();
-		exit_status = list_dir();
-	}
+	if (conf.autols == 1)
+		reload_dirlist();
 
-	return exit_status;
+	return FUNC_SUCCESS;
 }
 
 /* Go back one entry in dirhist */
@@ -1097,13 +747,13 @@ int
 back_function(char **args)
 {
 	if (!args)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	if (args[1]) {
 		if (!IS_HELP(args[1]))
 			return surf_hist(args);
 		puts(_(BACK_USAGE));
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	/* Find the previous non-consecutive equal and valid entry */
@@ -1115,21 +765,21 @@ back_function(char **args)
 	}
 
 	if (i < 0)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	dirhist_cur_index = i;
 
-	if (xchdir(old_pwd[dirhist_cur_index], SET_TITLE) == EXIT_SUCCESS)
+	if (xchdir(old_pwd[dirhist_cur_index], SET_TITLE) == FUNC_SUCCESS)
 		return set_path(old_pwd[dirhist_cur_index]);
 
-	xerror("cd: %s: %s\n", old_pwd[dirhist_cur_index], strerror(errno));
+	xerror("cd: '%s': %s\n", old_pwd[dirhist_cur_index], strerror(errno));
 
 	/* Invalidate this entry */
 	*old_pwd[dirhist_cur_index] = KEY_ESC;
 	if (dirhist_cur_index > 0)
 		dirhist_cur_index--;
 
-	return EXIT_FAILURE;
+	return FUNC_FAILURE;
 }
 
 /* Go forth one entry in dirhist */
@@ -1137,13 +787,13 @@ int
 forth_function(char **args)
 {
 	if (!args)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	if (args[1]) {
 		if (!IS_HELP(args[1]))
 			return surf_hist(args);
 		puts(_(FORTH_USAGE));
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	/* Find the next valid entry */
@@ -1155,15 +805,15 @@ forth_function(char **args)
 	}
 
 	if (i > dirhist_total_index)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	dirhist_cur_index = i;
 
 
-	if (xchdir(old_pwd[dirhist_cur_index], SET_TITLE) == EXIT_SUCCESS)
+	if (xchdir(old_pwd[dirhist_cur_index], SET_TITLE) == FUNC_SUCCESS)
 		return set_path(old_pwd[dirhist_cur_index]);
 
-	xerror("cd: %s: %s\n", old_pwd[dirhist_cur_index], strerror(errno));
+	xerror("cd: '%s': %s\n", old_pwd[dirhist_cur_index], strerror(errno));
 
 	/* Invalidate this entry */
 	*old_pwd[dirhist_cur_index] = KEY_ESC;
@@ -1171,5 +821,5 @@ forth_function(char **args)
 	&& old_pwd[dirhist_cur_index + 1])
 		dirhist_cur_index++;
 
-	return EXIT_FAILURE;
+	return FUNC_FAILURE;
 }
