@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -38,7 +38,6 @@
 
 #include "aux.h"
 #include "colors.h" /* update_warning_prompt_text_color() */
-#include "exec.h"
 #include "file_operations.h"
 #include "history.h"
 #include "init.h"
@@ -48,67 +47,16 @@
 #include "navigation.h"
 #include "prompt.h"
 #include "sanitize.h"
-
+#include "spawn.h"
 #ifndef _NO_SUGGESTIONS
 # include "suggestions.h"
 #endif /* !_NO_SUGGESTIONS */
 
-#define CTLESC '\001'
-#define CTLNUL '\177'
-
-#define ROOT_IND "\001\x1b[1;31m\002R\001\x1b[0m\002"
-#define ROOT_IND_NO_COLOR "R"
-#define ROOT_IND_SIZE 17
-#define RDONLY_IND "RO\001\x1b[0m\002"
-#define RDONLY_IND_SIZE (MAX_COLOR + 8 + 1)
-#define STEALTH_IND "S\001\x1b[0m\002"
-#define STEALTH_IND_SIZE (MAX_COLOR + 7 + 1)
-
-#define EMERGENCY_PROMPT_MSG "Error decoding prompt line. Using an \
-emergency prompt"
-#define EMERGENCY_PROMPT "\001\x1b[0m\002> "
-
-/* Flag macros to generate files statistic string for the prompt */
-#define STATS_DIR      0
-#define STATS_REG      1
-#define STATS_EXE      2
-#define STATS_HIDDEN   3
-#define STATS_SUID     4
-#define STATS_SGID     5
-#define STATS_FIFO     6
-#define STATS_SOCK     7
-#define STATS_BLK      8
-#define STATS_CHR      9
-#define STATS_CAP      10
-#define STATS_LNK      11
-#define STATS_BROKEN_L 12 /* Broken link */
-#define STATS_MULTI_L  13 /* Multi-link */
-#define STATS_OTHER_W  14 /* Other writable */
-#define STATS_STICKY   15
-#define STATS_EXTENDED 16 /* Extended attributes (acl) */
-#define STATS_UNKNOWN  17
-#define STATS_UNSTAT   18
-#ifdef SOLARIS_DOORS
-# define STATS_DOOR    19
-# define STATS_PORT    20
-#endif /* SOLARIS_DOORS */
-
-#define NOTIF_SEL     0
-#define NOTIF_TRASH   1
-#define NOTIF_WARNING 2
-#define NOTIF_ERROR   3
-#define NOTIF_NOTICE  4
-#define NOTIF_ROOT    5
-
-/* Size of the indicator for msgs, trash, and sel */
-#define N_IND MAX_COLOR + 1 + sizeof(size_t) + 6 + 1 + 13
-/* Color + 1 letter + plus unsigned integer + RL_NC size + nul char */
-
 static inline char *
 gen_time(const int c)
 {
 	char *temp = (char *)NULL;
-	time_t rawtime = time(NULL);
+	const time_t rawtime = time(NULL);
 	struct tm tm;
 
 	if (!localtime_r(&rawtime, &tm)) {
@@ -141,8 +89,8 @@ gen_time(const int c)
 static inline char *
 get_dir_basename(const char *_path)
 {
-	char *temp = (char *)NULL,
-		 *ret = (char *)NULL;
+	char *temp = (char *)NULL;
+	char *ret = (char *)NULL;
 
 	/* If not root dir (/), get last path component */
 	if (!(*_path == '/' && !*(_path + 1)))
@@ -235,10 +183,10 @@ gen_workspace(void)
 static inline char *
 gen_exit_status(void)
 {
-	size_t code_len = (size_t)DIGINUM(exit_code);
-	size_t len = code_len + 3 + (MAX_COLOR * 2);
+	const size_t code_len = (size_t)DIGINUM(exit_code);
+	const size_t len = code_len + 3 + (MAX_COLOR * 2);
 
-	char *temp = (char *)xnmalloc(len, sizeof(char));
+	char *temp = xnmalloc(len, sizeof(char));
 	snprintf(temp, len, "%s%d\001%s\002",
 		(exit_code == 0) ? (conf.colorize == 1 ? xs_c : "")
 		: (conf.colorize == 1 ? xf_c : ""), exit_code, df_c);
@@ -252,7 +200,7 @@ gen_escape_char(char **line, int *c)
 	(*line)++;
 	*c = 0;
 	/* 27 (dec) == 033 (octal) == 0x1b (hex) == \e */
-	char *temp = (char *)xnmalloc(2, sizeof(char));
+	char *temp = xnmalloc(2, sizeof(char));
 	*temp = '\033';
 	temp[1] = '\0';
 
@@ -263,13 +211,12 @@ static inline char *
 gen_octal(char **line, int *c)
 {
 	char octal_string[4];
-	int n;
 
 	xstrsncpy(octal_string, *line, sizeof(octal_string));
 	octal_string[3] = '\0';
 
-	n = read_octal(octal_string);
-	char *temp = (char *)xnmalloc(3, sizeof(char));
+	const int n = read_octal(octal_string);
+	char *temp = xnmalloc(3, sizeof(char));
 
 	if (n == CTLESC || n == CTLNUL) {
 		*line += 3;
@@ -334,7 +281,7 @@ gen_hostname(const int c)
 static inline char *
 gen_user_flag(void)
 {
-	char *temp = (char *)xnmalloc(2, sizeof(char));
+	char *temp = xnmalloc(2, sizeof(char));
 	*temp = user.uid == 0 ? '#' : '$';
 	temp[1] = '\0';
 
@@ -344,7 +291,7 @@ gen_user_flag(void)
 static inline char *
 gen_mode(void)
 {
-	char *temp = (char *)xnmalloc(2, sizeof(char));
+	char *temp = xnmalloc(2, sizeof(char));
 
 	if (conf.light_mode == 1) {
 		*temp = 'L';
@@ -359,7 +306,7 @@ gen_mode(void)
 static inline char*
 gen_misc(const int c)
 {
-	char *temp = (char *)xnmalloc(2, sizeof(char));
+	char *temp = xnmalloc(2, sizeof(char));
 
 	if (c == 'n')
 		*temp = '\n';
@@ -376,7 +323,7 @@ gen_misc(const int c)
 static inline char *
 gen_non_print_sequence(const int c)
 {
-	char *temp = (char *)xnmalloc(2, sizeof(char));
+	char *temp = xnmalloc(2, sizeof(char));
 	*temp = (c == '[') ? RL_PROMPT_START_IGNORE : RL_PROMPT_END_IGNORE;
 	temp[1] = '\0';
 
@@ -403,12 +350,12 @@ add_string(char **tmp, const int c, char **line, char **res, size_t *len)
 
 	*len += strlen(*tmp);
 
-	size_t l = *len + 2	+ (wrong_cmd ? (MAX_COLOR + 6) : 0);
+	const size_t l = *len + 2 + (wrong_cmd ? (MAX_COLOR + 6) : 0);
 	if (!*res) {
-		*res = (char *)xnmalloc(l + 1, sizeof(char));
+		*res = xnmalloc(l + 1, sizeof(char));
 		*(*res) = '\0';
 	} else {
-		*res = (char *)xrealloc(*res, (l + 1) * sizeof(char));
+		*res = xnrealloc(*res, l + 1, sizeof(char));
 	}
 
 	xstrncat(*res, strlen(*res), *tmp, l + 1);
@@ -428,11 +375,11 @@ reset_ifs(const char *value)
 static inline void
 substitute_cmd(char **line, char **res, size_t *len)
 {
-	int tmp = strcntchr(*line, ')');
+	const int tmp = strcntchr(*line, ')');
 	if (tmp == -1) return; /* No ending bracket */
 
-	size_t tmp_len = strlen(*line) + 2;
-	char *tmp_str = (char *)xnmalloc(tmp_len, sizeof(char));
+	const size_t tmp_len = strlen(*line) + 2;
+	char *tmp_str = xnmalloc(tmp_len, sizeof(char));
 	snprintf(tmp_str, tmp_len, "$%s", *line);
 
 	tmp_str[tmp + 2] = '\0';
@@ -442,7 +389,7 @@ substitute_cmd(char **line, char **res, size_t *len)
 	setenv("IFS", "", 1);
 
 	wordexp_t wordbuf;
-	if (wordexp(tmp_str, &wordbuf, 0) != EXIT_SUCCESS) {
+	if (wordexp(tmp_str, &wordbuf, 0) != FUNC_SUCCESS) {
 		free(tmp_str);
 		reset_ifs(old_value);
 		return;
@@ -454,10 +401,10 @@ substitute_cmd(char **line, char **res, size_t *len)
 		for (size_t j = 0; j < wordbuf.we_wordc; j++) {
 			*len += strlen(wordbuf.we_wordv[j]);
 			if (!*res) {
-				*res = (char *)xnmalloc(*len + 2, sizeof(char));
+				*res = xnmalloc(*len + 2, sizeof(char));
 				*(*res) = '\0';
 			} else {
-				*res = (char *)xrealloc(*res, (*len + 2) * sizeof(char));
+				*res = xnrealloc(*res, *len + 2, sizeof(char));
 			}
 			xstrncat(*res, strlen(*res), wordbuf.we_wordv[j], *len + 2);
 		}
@@ -476,6 +423,7 @@ gen_emergency_prompt(void)
 		f = 1;
 		xerror("%s: %s\n", PROGRAM_NAME, EMERGENCY_PROMPT_MSG);
 	}
+
 	char *_prompt = savestring(EMERGENCY_PROMPT, sizeof(EMERGENCY_PROMPT) - 1);
 	return _prompt;
 }
@@ -514,10 +462,10 @@ gen_stats_str(const int flag)
 
 	char *p = (char *)NULL;
 	if (val != 0) {
-		p = (char *)xnmalloc(32, sizeof(char));
+		p = xnmalloc(32, sizeof(char));
 		snprintf(p, 32, "%zu", val);
 	} else {
-		p = (char *)xnmalloc(2, sizeof(char));
+		p = xnmalloc(2, sizeof(char));
 		*p = '-';
 		p[1] = '\0';
 	}
@@ -528,10 +476,9 @@ gen_stats_str(const int flag)
 static inline char *
 gen_notification(const int flag)
 {
-	char *p;
-	size_t len = 32;
+	const size_t len = 32;
 
-	p = (char *)xnmalloc(len, sizeof(char));
+	char *p = xnmalloc(len, sizeof(char));
 	*p = '\0';
 
 	switch (flag) {
@@ -556,8 +503,8 @@ gen_notification(const int flag)
 			snprintf(p, len, "*%zu", sel_n);
 		break;
 	case NOTIF_TRASH:
-		if (trash_n > 2)
-			snprintf(p, len, "T%zu", trash_n - 2);
+		if (trash_n > 0)
+			snprintf(p, len, "T%zu", trash_n);
 		break;
 	default: break;
 	}
@@ -571,7 +518,7 @@ gen_nesting_level(const int mode)
 	char *p = (char *)NULL;
 
 	if (mode == 'i') {
-		p = (char *)xnmalloc(32, sizeof(char));
+		p = xnmalloc(32, sizeof(char));
 		snprintf(p, 32, "%d", nesting_level);
 		return p;
 	}
@@ -583,8 +530,8 @@ gen_nesting_level(const int mode)
 		return p;
 	}
 
-	size_t len = (MAX_COLOR * 2) + 32;
-	p = (char *)xnmalloc(len, sizeof(char));
+	const size_t len = (MAX_COLOR * 2) + 32;
+	p = xnmalloc(len, sizeof(char));
 	snprintf(p, len, "(%d)", nesting_level);
 
 	return p;
@@ -598,7 +545,8 @@ decode_prompt(char *line)
 	if (!line)
 		return (char *)NULL;
 
-	char *temp = (char *)NULL, *result = (char *)NULL;
+	char *temp = (char *)NULL;
+	char *result = (char *)NULL;
 	size_t result_len = 0;
 	int c;
 
@@ -737,9 +685,9 @@ ADD_STRING:
 			}
 #endif /* !__HAIKU__ && !__OpenBSD__ && !__ANDROID__ */
 
-			size_t new_len = result_len + 2
-							+ (wrong_cmd ? (MAX_COLOR + 6) : 0);
-			result = (char *)xrealloc(result, new_len * sizeof(char));
+			const size_t new_len = result_len + 2
+				+ (wrong_cmd ? (MAX_COLOR + 6) : 0);
+			result = xnrealloc(result, new_len, sizeof(char));
 			result[result_len] = (char)c;
 			result_len++;
 			result[result_len] = '\0';
@@ -761,20 +709,28 @@ ADD_STRING:
 static inline void
 check_cwd(void)
 {
-	while (xchdir(workspaces[cur_ws].path, SET_TITLE) != EXIT_SUCCESS) {
+	int cwd_change = 0;
+
+	while (xchdir(workspaces[cur_ws].path, SET_TITLE) != FUNC_SUCCESS) {
 		char *ret = strrchr(workspaces[cur_ws].path, '/');
-		if (ret && ret != workspaces[cur_ws].path)
+		if (ret && ret != workspaces[cur_ws].path) {
 			*ret = '\0';
-		else
+			cwd_change = 1;
+		} else {
 			break;
+		}
 	}
+
+	if (cwd_change == 1 && conf.autols == 1)
+		refresh_screen();
 }
 
 /* Remove all final slash(es) from path, if any */
 static inline void
 trim_final_slashes(void)
 {
-	size_t path_len = strlen(workspaces[cur_ws].path), i;
+	const size_t path_len = strlen(workspaces[cur_ws].path);
+	size_t i;
 
 	for (i = path_len - 1; workspaces[cur_ws].path[i] && i > 0; i--) {
 		if (workspaces[cur_ws].path[i] != '/')
@@ -821,14 +777,16 @@ run_prompt_cmds(void)
 	if (conf.ext_cmd_ok == 0 || prompt_cmds_n == 0)
 		return;
 
-	int tflags = flags;
+	const int tflags = flags;
 	flags &= ~DELAYED_REFRESH;
 	size_t i;
+
 	for (i = 0; i < prompt_cmds_n; i++) {
 		if (xargs.secure_cmds == 0
-		|| sanitize_cmd(prompt_cmds[i], SNT_PROMPT) == EXIT_SUCCESS)
+		|| sanitize_cmd(prompt_cmds[i], SNT_PROMPT) == FUNC_SUCCESS)
 			launch_execl(prompt_cmds[i]);
 	}
+
 	flags = tflags;
 }
 
@@ -850,11 +808,11 @@ update_trash_indicator(void)
 
 	trash_files_dir_mtime = a.st_mtime;
 
-	filesn_t n = count_dir(trash_files_dir, NO_CPOP);
+	const filesn_t n = count_dir(trash_files_dir, NO_CPOP);
 	if (n <= 2)
 		trash_n = 0;
 	else
-		trash_n = (size_t)n;
+		trash_n = (size_t)n - 2;
 }
 #endif /* !_NO_TRASH */
 
@@ -871,7 +829,7 @@ setenv_prompt(void)
 	snprintf(t, sizeof(t), "%d", (int)sel_n);
 	setenv("CLIFM_STAT_SEL", t, 1);
 #ifndef _NO_TRASH
-	snprintf(t, sizeof(t), "%d", trash_n > 2 ? (int)trash_n - 2 : 0);
+	snprintf(t, sizeof(t), "%d", trash_n > 0 ? (int)trash_n : 0);
 	setenv("CLIFM_STAT_TRASH", t, 1);
 #endif /* !_NO_TRASH */
 	snprintf(t, sizeof(t), "%d", (int)msgs.error);
@@ -927,15 +885,15 @@ construct_prompt(const char *decoded_prompt)
 			snprintf(warn_ind, N_IND, "%sW%zu%s", wm_c, msgs.warning, RL_NC);
 		if (msgs.notice > 0)
 			snprintf(notice_ind, N_IND, "%sN%zu%s", nm_c, msgs.notice, RL_NC);
-		if (trash_n > 2)
+		if (trash_n > 0)
 			snprintf(trash_ind, N_IND, "%sT%zu%s",
-				ti_c, (size_t)trash_n - 2, RL_NC);
+				ti_c, (size_t)trash_n, RL_NC);
 		if (sel_n > 0)
 			snprintf(sel_ind, N_IND, "%s*%zu%s", li_c, sel_n, RL_NC);
 	}
 
-	size_t prompt_len = set_prompt_length(strlen(decoded_prompt));
-	char *the_prompt = (char *)xnmalloc(prompt_len, sizeof(char));
+	const size_t prompt_len = set_prompt_length(strlen(decoded_prompt));
+	char *the_prompt = xnmalloc(prompt_len, sizeof(char));
 
 	if (prompt_notif == 1) {
 		snprintf(the_prompt, prompt_len,
@@ -953,7 +911,8 @@ construct_prompt(const char *decoded_prompt)
 			(sel_n > 0) ? sel_ind : "",
 			decoded_prompt, RL_NC, tx_c);
 	} else {
-		snprintf(the_prompt, prompt_len, "%s%s\001%s\002", decoded_prompt, RL_NC, tx_c);
+		snprintf(the_prompt, prompt_len, "%s%s\001%s\002",
+			decoded_prompt, RL_NC, tx_c);
 	}
 
 	return the_prompt;
@@ -1054,21 +1013,21 @@ expand_history(char **input)
 	char *hist_c = strchr(*input, history_expansion_char);
 	if (!hist_c || (hist_c != *input && *(hist_c - 1) != ' ')
 	|| exclude_from_history(*input) == 1)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	char *exp_input = (char *)NULL;
-	int ret = history_expand(*input, &exp_input);
+	const int ret = history_expand(*input, &exp_input);
 
 	if (ret == -1) { /* Error in expansion */
 		xerror("%s: %s\n", PROGRAM_NAME, exp_input);
 		free(*input);
 		free(exp_input);
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	if (ret == 0) { /* No expansion took place */
 		free(exp_input);
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	printf("%s\n", exp_input);
@@ -1082,7 +1041,7 @@ expand_history(char **input)
 	/* (ret == 1) Display and execute */
 	*input = exp_input;
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static char *
@@ -1142,7 +1101,7 @@ prompt(const int prompt_flag)
 
 	flags &= ~DELAYED_REFRESH;
 
-	if (expand_history(&input) != EXIT_SUCCESS)
+	if (expand_history(&input) != FUNC_SUCCESS)
 		return (char *)NULL;
 
 	log_and_record(input);
@@ -1155,7 +1114,7 @@ list_prompts(void)
 {
 	if (prompts_n == 0) {
 		puts(_("prompt: No extra prompts found. Using the default prompt"));
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	size_t i;
@@ -1170,7 +1129,7 @@ list_prompts(void)
 			printf("  %s\n", prompts[i].name);
 	}
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
@@ -1189,31 +1148,31 @@ switch_prompt(const size_t n)
 	prompt_notif = prompts[n].notifications;
 
 	if (xargs.warning_prompt == 0)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	conf.warning_prompt = prompts[n].warning_prompt_enabled;
 
 	update_warning_prompt_text_color();
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
 set_prompt(char *name)
 {
 	if (!name || !*name)
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
 	if (prompts_n == 0) {
 		xerror("%s\n", _("prompt: No extra prompts defined. Using the "
 			"default prompt"));
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
-	char *p = dequote_str(name, 0);
+	char *p = unescape_str(name, 0);
 	if (!p) {
-		xerror(_("prompt: %s: Error dequoting string\n"), name);
-		return EXIT_FAILURE;
+		xerror(_("prompt: %s: Error unescaping string\n"), name);
+		return FUNC_FAILURE;
 	}
 
 	int i = (int)prompts_n;
@@ -1227,7 +1186,7 @@ set_prompt(char *name)
 
 	xerror(_("prompt: %s: No such prompt\n"), p);
 	free(p);
-	return EXIT_FAILURE;
+	return FUNC_FAILURE;
 }
 
 static int
@@ -1239,7 +1198,7 @@ set_default_prompt(void)
 	conf.wprompt_str = savestring(DEF_WPROMPT_STR, sizeof(DEF_WPROMPT_STR) - 1);
 	*cur_prompt_name = '\0';
 	prompt_notif = DEF_PROMPT_NOTIF;
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
@@ -1247,23 +1206,23 @@ edit_prompts_file(char *app)
 {
 	if (xargs.stealth_mode == 1) {
 		printf("%s: prompt: %s\n", PROGRAM_NAME, STEALTH_DISABLED);
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	if (!prompts_file || !*prompts_file) {
-		xerror("%s\n", _("prompt: No prompts file found"));
-		return EXIT_FAILURE;
+		xerror("%s\n", _("prompt: Prompts file not found"));
+		return FUNC_FAILURE;
 	}
 
 	struct stat a;
 	if (stat(prompts_file, &a) == -1) {
-		xerror("prompt: %s: %s\n", prompts_file, strerror(errno));
-		return EXIT_FAILURE;
+		xerror("prompt: '%s': %s\n", prompts_file, strerror(errno));
+		return errno;
 	}
 
-	time_t old_time = a.st_mtime;
+	const time_t old_time = a.st_mtime;
 
-	int ret = EXIT_FAILURE;
+	int ret = FUNC_FAILURE;
 	if (app && *app) {
 		char *cmd[] = {app, prompts_file, NULL};
 		ret = launch_execv(cmd, FOREGROUND, E_NOFLAG);
@@ -1273,19 +1232,26 @@ edit_prompts_file(char *app)
 		open_in_foreground = 0;
 	}
 
-	if (ret != EXIT_SUCCESS)
+	if (ret != FUNC_SUCCESS)
 		return ret;
 
-	stat(prompts_file, &a);
+	if (stat(prompts_file, &a) == -1) {
+		xerror("prompt: '%s': %s\n", prompts_file, strerror(errno));
+		return errno;
+	}
+
 	if (conf.autols == 1)
 		reload_dirlist();
+
 	if (old_time == a.st_mtime)
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 
 	ret = load_prompts();
 	print_reload_msg(_("File modified. Prompts reloaded\n"));
+
 	if (*cur_prompt_name)
 		set_prompt(cur_prompt_name);
+
 	return ret;
 }
 
@@ -1298,7 +1264,7 @@ prompt_function(char **args)
 
 	if (IS_HELP(args[0])) {
 		puts(PROMPT_USAGE);
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	if (*args[0] == 'u' && strcmp(args[0], "unset") == 0)
@@ -1308,10 +1274,10 @@ prompt_function(char **args)
 		return edit_prompts_file(args[1]);
 
 	if (*args[0] == 'r' && strcmp(args[0], "reload") == 0) {
-		int ret = load_prompts();
-		if (ret == EXIT_SUCCESS) {
+		const int ret = load_prompts();
+		if (ret == FUNC_SUCCESS) {
 			printf(_("%s: Prompts successfully reloaded\n"), PROGRAM_NAME);
-			return EXIT_SUCCESS;
+			return FUNC_SUCCESS;
 		}
 		return ret;
 	}
