@@ -94,9 +94,13 @@
 #include "checks.h" /* check_file_access(), is_number() */
 #include "colors.h" /* get_dir_color(), get_regfile_color() */
 #include "messages.h"
+#ifndef _NO_MAGIC
+# include "mime.h"  /* xmagic() */
+#endif /* !_NO_MAGIC */
 #include "misc.h"
 #include "properties.h"
 #include "readline.h"   /* Required by the 'pc' command */
+#include "xdu.h" /* dir_info(), dir_size() */
 
 /* A few macros for nano-second precision.
  * Used to print timestamps with the 'p/pp' command. */
@@ -133,7 +137,7 @@ static int
 print_file_attrs(const int aflags)
 {
 	if (aflags == -1) {
-		puts(_("Unavailable"));
+		puts(_("unavailable"));
 		return FUNC_FAILURE;
 	}
 
@@ -227,43 +231,6 @@ get_link_color(const char *name)
 	return color;
 }
 
-static off_t
-get_total_size(char *filename, int *status)
-{
-	off_t total_size = 0;
-
-	char _path[PATH_MAX + 1]; *_path = '\0';
-	snprintf(_path, sizeof(_path), "%s/", filename);
-
-	fputs(_("Total size: \t"), stdout);
-
-#define SCANNING_MSG "Scanning..."
-	if (term_caps.suggestions == 1) {
-		HIDE_CURSOR;
-		fputs(dn_c, stdout);
-		fputs(SCANNING_MSG, stdout);
-		fflush(stdout);
-	}
-	fflush(stdout);
-
-#ifdef USE_XDU
-	total_size = dir_size(*_path ? _path : filename, 1, status);
-#else
-	total_size = dir_size(*_path ? _path : filename,
-		(bin_flags & (GNU_DU_BIN_DU | GNU_DU_BIN_GDU)) ? 1 : 0, status);
-#endif /* USE_XDU */
-
-	if (term_caps.suggestions == 1) {
-		MOVE_CURSOR_LEFT((int)sizeof(SCANNING_MSG) - 1);
-		ERASE_TO_RIGHT;
-		UNHIDE_CURSOR;
-		fputs(df_c, stdout);
-		fflush(stdout);
-	}
-
-	return total_size;
-}
-
 static struct perms_t
 set_invalid_file_perms(void)
 {
@@ -1115,14 +1082,21 @@ print_file_name(char *filename, const char *color, const int follow_link,
 	char *wname = wc_xstrlen(filename) == 0
 		? replace_invalid_chars(filename) : (char *)NULL;
 
+	char name[(NAME_MAX * sizeof(wchar_t)) + 3]; *name = '\0';
+	if (strchr(wname ? wname : filename, ' '))
+		snprintf(name, sizeof(name), "'%s'", wname ? wname : filename);
+
+	char *n = *name ? name : (wname ? wname : filename);
+
 	if (follow_link == 1) { /* 'pp' command */
 		if (link_target && *link_target) {
-			printf(_("\tName: %s%s%s <- %s%s%s\n"), color,
-				link_target, df_c, ln_c,
-				wname ? wname : filename, df_c);
+			char t[PATH_MAX * sizeof(wchar_t) + 3]; *t = '\0';
+			if (strchr(link_target, ' '))
+				snprintf(t, sizeof(t), "'%s'", link_target);
+			printf(_("\tName: %s%s%s %s<-%s %s%s%s\n"), color,
+				*t ? t : link_target, df_c, dn_c, df_c, ln_c, n, df_c);
 		} else {
-			printf(_("\tName: %s%s%s\n"), color,
-				wname ? wname : filename, df_c);
+			printf(_("\tName: %s%s%s\n"), color, n, df_c);
 		}
 
 		free(wname);
@@ -1131,7 +1105,7 @@ print_file_name(char *filename, const char *color, const int follow_link,
 
 	/* 'p' command */
 	if (!S_ISLNK(mode)) {
-		printf(_("\tName: %s%s%s\n"), color, wname ? wname : filename, df_c);
+		printf(_("\tName: %s%s%s\n"), color, n, df_c);
 		free(wname);
 		return;
 	}
@@ -1140,20 +1114,23 @@ print_file_name(char *filename, const char *color, const int follow_link,
 	const ssize_t tlen =
 		xreadlink(XAT_FDCWD, filename, target, sizeof(target));
 
+	char t[PATH_MAX * sizeof(wchar_t) + 3]; *t = '\0';
+	if (tlen != -1 && *target && strchr(target, ' '))
+		snprintf(t, sizeof(t), "'%s'", target);
+
 	struct stat a;
 	if (tlen != -1 && *target && lstat(target, &a) != -1) {
 		char *link_color = get_link_color(target);
-
-		printf(_("\tName: %s%s%s -> %s%s%s\n"), ln_c, wname ? wname
-			: filename, df_c, link_color, target, df_c);
+		printf(_("\tName: %s%s%s %s->%s %s%s%s\n"), ln_c, n, df_c,
+			dn_c, df_c, link_color, *t ? t : target, df_c);
 
 	} else { /* Broken link */
 		if (tlen != -1 && *target) {
-			printf(_("\tName: %s%s%s -> %s%s%s (broken link)\n"), or_c,
-				wname ? wname : filename, df_c, uf_c, target, df_c);
+			printf(_("\tName: %s%s%s %s->%s %s%s%s (broken link)\n"), or_c,
+				n, df_c, dn_c, df_c, uf_c, *t ? t : target, df_c);
 		} else {
-			printf(_("\tName: %s%s%s -> ???\n"), or_c, wname ? wname
-				: filename, df_c);
+			printf(_("\tName: %s%s%s %s-> ???%s\n"), or_c, n, df_c,
+				dn_c, df_c);
 		}
 	}
 
@@ -1296,8 +1273,8 @@ print_file_details(char *filename, const struct stat *attr, const char file_type
 	case REG_PCHR: fputs(_("Regular file"), stdout); break;
 	case DIR_PCHR: fputs(_("Directory"), stdout); break;
 	case LNK_PCHR: fputs(_("Symbolic link"), stdout); break;
-	case FIFO_PCHR: fputs(_("Fifo"), stdout); break;
-	case SOCK_PCHR: fputs(_("Socket"), stdout); break;
+	case FIFO_PCHR: fputs(_("Fifo    "), stdout); break;
+	case SOCK_PCHR: fputs(_("Socket  "), stdout); break;
 	case BLKDEV_PCHR: fputs(_("Block special file"), stdout); break;
 	case CHARDEV_PCHR: fputs(_("Character special file"), stdout); break;
 #ifndef _BE_POSIX
@@ -1306,8 +1283,8 @@ print_file_details(char *filename, const struct stat *attr, const char file_type
 	case ARCH2_PCHR: fputs(_("Archive state 2"), stdout); break;
 # endif /* S_ARCH1 */
 # ifdef SOLARIS_DOORS
-	case DOOR_PCHR: fputs(_("Door"), stdout); break;
-	case PORT_PCHR: fputs(_("Port"), stdout); break;
+	case DOOR_PCHR: fputs(_("Door    "), stdout); break;
+	case PORT_PCHR: fputs(_("Port    "), stdout); break;
 # endif /* SOLARIS_DOORS */
 # ifdef S_IFWHT
 	case WHT_PCHR: fputs(_("Whiteout"), stdout); break;
@@ -1585,6 +1562,72 @@ print_timestamps(char *filename, const struct stat *attr)
 #endif /* ST_BTIME */
 }
 
+static int
+err_no_file(const char *filename, const char *target, const int errnum)
+{
+	char *errname = xargs.stat > 0 ? PROGRAM_NAME : "prop";
+
+	if (*target) {
+		xerror(_("%s: %s %s->%s %s: Broken symbolic link\n"),
+			errname, filename, mi_c, df_c, target);
+	} else {
+		xerror("%s: '%s': %s\n", errname, filename, strerror(errnum));
+	}
+
+	return errnum;
+}
+
+static void
+print_file_mime(const char *name)
+{
+#ifdef _NO_MAGIC
+	UNUSED(name);
+	return;
+#else
+	fputs("MIME type:\t", stdout);
+	char *n = xmagic(name, 1);
+	if (n) {
+		printf("%s\n", n);
+		free(n);
+	} else {
+		printf("%s%c%s\n", dn_c, UNKNOWN_CHR, df_c);
+	}
+#endif /* _NO_MAGIC */
+}
+
+#ifdef USE_DU1
+static off_t
+get_total_size(char *filename, int *status)
+{
+	off_t total_size = 0;
+
+	char _path[PATH_MAX + 1]; *_path = '\0';
+	snprintf(_path, sizeof(_path), "%s/", filename);
+
+	fputs(_("Total size: \t"), stdout);
+
+#define SCANNING_MSG "Scanning..."
+	if (term_caps.suggestions == 1) {
+		HIDE_CURSOR;
+		fputs(dn_c, stdout);
+		fputs(SCANNING_MSG, stdout);
+		fflush(stdout);
+	}
+	fflush(stdout);
+
+	total_size = dir_size(*_path ? _path : filename, 1, status);
+
+	if (term_caps.suggestions == 1) {
+		MOVE_CURSOR_LEFT((int)sizeof(SCANNING_MSG) - 1);
+		ERASE_TO_RIGHT;
+		UNHIDE_CURSOR;
+		fputs(df_c, stdout);
+		fflush(stdout);
+	}
+
+	return total_size;
+}
+
 static void
 print_file_size(char *filename, const struct stat *attr, const int file_perm,
 	const int full_dirsize)
@@ -1621,7 +1664,7 @@ print_file_size(char *filename, const struct stat *attr, const int file_perm,
 		return;
 	}
 
-	if (full_dirsize == 0) /* We're running 'p', not 'pp'. */
+	if (full_dirsize == 0) // We're running 'p', not 'pp'.
 		return;
 
 	int du_status = 0;
@@ -1629,9 +1672,9 @@ print_file_size(char *filename, const struct stat *attr, const int file_perm,
 		? get_total_size(filename, &du_status) : (-2);
 
 	if (total_size < 0) {
-		if (total_size == -2) /* No access */
+		if (total_size == -2) // No access
 			printf(_("Total size: \t%s%c%s\n"), dn_c, UNKNOWN_CHR, cend);
-		else /* get_total_size returned error (-1) */
+		else // get_total_size returned error (-1)
 			puts(UNKNOWN_STR);
 		return;
 	}
@@ -1649,12 +1692,10 @@ print_file_size(char *filename, const struct stat *attr, const int file_perm,
 		return;
 	}
 
-#ifndef USE_XDU
 	if (bin_flags & (GNU_DU_BIN_DU | GNU_DU_BIN_GDU)) {
-#endif // USE_XDU
-		char err[sizeof(xf_c) + 6]; *err = '\0';
+		char err[sizeof(xf_cb) + 6]; *err = '\0';
 		if (du_status != 0)
-			snprintf(err, sizeof(err), "%s%c%s", xf_c, DU_ERR_CHAR, NC);
+			snprintf(err, sizeof(err), "%s%c%s", xf_cb, DU_ERR_CHAR, NC);
 
 		printf("%s%s%s%s ", err, csize, human_size, cend);
 
@@ -1663,83 +1704,9 @@ print_file_size(char *filename, const struct stat *attr, const int file_perm,
 
 		printf("(%s%s)\n", conf.apparent_size == 1 ? _("apparent")
 			: _("disk usage"), xargs.si == 1 ? ",si" : "");
-#ifndef USE_XDU
 	} else {
 		printf("%s%s%s\n", csize, human_size, cend);
 	}
-#endif // USE_XDU
-}
-
-static int
-err_no_file(const char *filename, const char *target, const int errnum)
-{
-	char *errname = xargs.stat > 0 ? PROGRAM_NAME : "prop";
-
-	if (*target) {
-		xerror(_("%s: %s %s->%s %s: Broken symbolic link\n"),
-			errname, filename, mi_c, df_c, target);
-	} else {
-		xerror("%s: '%s': %s\n", errname, filename, strerror(errnum));
-	}
-
-	return errnum;
-}
-
-#ifdef CLIFM_DIR_INFO
-struct dir_info_t {
-	unsigned long long dirs;
-	unsigned long long files;
-	unsigned long long links;
-};
-
-/* Recursively count files and directories in the directory DIR and store
- * values in the INFO struct. If a directory cannot be read, or a file cannot
- * be stat'ed, STATUS is set to the appropriate errno value. */
-void
-dir_info(const char *dir, int *status, struct dir_info_t *info)
-{
-	if (!dir || !*dir) {
-		*status = ENOENT;
-		return;
-	}
-
-	struct stat a;
-	DIR *p;
-
-	if ((p = opendir(dir)) == NULL) {
-		*status = errno;
-		return;
-	}
-
-	struct dirent *ent;
-
-	while ((ent = readdir(p)) != NULL) {
-		if (SELFORPARENT(ent->d_name))
-			continue;
-
-		char buf[PATH_MAX + 1];
-		snprintf(buf, sizeof(buf), "%s/%s", dir, ent->d_name);
-
-		if (lstat(buf, &a) == -1) {
-			*status = errno;
-			/* We cannot know if the file is a directory. Let's take it as
-			 * a plain file. */
-			info->files++;
-			continue;
-		}
-
-		if (S_ISDIR(a.st_mode)) {
-			info->dirs++;
-			dir_info(buf, status, info);
-		} else if (S_ISLNK(a.st_mode)) {
-			info->links++;
-		} else {
-			info->files++;
-		}
-	}
-
-	closedir(p);
-	return;
 }
 
 static void
@@ -1753,7 +1720,6 @@ print_dir_items(const char *dir, const int file_perm)
 	}
 
 	struct dir_info_t info = {0};
-	int status = 0;
 
 #define COUNTING_MSG "Counting..."
 	if (term_caps.suggestions == 1) {
@@ -1763,7 +1729,7 @@ print_dir_items(const char *dir, const int file_perm)
 	}
 	fflush(stdout);
 
-	dir_info(dir, &status, &info);
+	dir_info(dir, 1, &info);
 
 	if (term_caps.suggestions == 1) {
 		MOVE_CURSOR_LEFT((int)sizeof(COUNTING_MSG) - 1);
@@ -1774,40 +1740,113 @@ print_dir_items(const char *dir, const int file_perm)
 	}
 
 	char read_err[5 + (MAX_COLOR * 2)]; *read_err = '\0';
-	if (status != 0)
-		snprintf(read_err, sizeof(read_err), "%s%c%s", xf_c, DU_ERR_CHAR, df_c);
+	if (info.status != 0)
+		snprintf(read_err, sizeof(read_err), "%s%c%s", xf_cb, DU_ERR_CHAR, df_c);
 
-	printf(_("%s%s%llu%s (%s%llu%s directories, %s%llu%s files, %s%llu%s links)\n"),
+	printf("%s%s%llu%s (%s%llu%s %s, %s%llu%s %s, %s%llu%s %s)\n",
 		read_err, BOLD, info.dirs + info.files + info.links, df_c,
-		BOLD, info.dirs, df_c,
-		BOLD, info.files, df_c,
-		BOLD, info.links, df_c);
+		BOLD, info.dirs, df_c, info.dirs == 1 ? _("directory") : _("directories"),
+		BOLD, info.files, df_c, info.files == 1 ? _("file") : _("files"),
+		BOLD, info.links, df_c, info.links == 1 ? _("link") : _("links"));
 }
-#endif /* CLIFM_DIR_INFO */
 
-#ifdef CLIFM_PROP_MIME
-#ifndef _NO_MAGIC
-# include "mime.h"
-#endif
+#else /* !USE_DU1 */
+static void
+print_file_size(const struct stat *attr)
+{
+	const off_t size =
+		(FILE_TYPE_NON_ZERO_SIZE(attr->st_mode) || S_TYPEISSHM(attr)
+		|| S_TYPEISTMO(attr)) ? FILE_SIZE_PTR(attr) : 0;
+
+	char *size_unit = construct_human_size(size);
+	char *csize = dz_c;
+	char *cend = conf.colorize == 1 ? df_c : "";
+
+	char sf[MAX_SHADE_LEN];
+	*sf = '\0';
+	if (conf.colorize == 1 && !*dz_c) {
+		get_color_size(size, sf, sizeof(sf));
+		csize = sf;
+	}
+
+	printf(_("Size: \t\t%s%s%s"), csize, size_unit ? size_unit : "?", cend);
+
+	const int bigger_than_bytes = size > (xargs.si == 1 ? 1000 : 1024);
+
+	if (bigger_than_bytes == 1)
+		printf(" / %s%jd B%s", csize, (intmax_t)size, cend);
+
+	printf(" (%s%s%s)\n", conf.apparent_size == 1 ? _("apparent")
+		: _("disk usage"), (xargs.si == 1 && bigger_than_bytes == 1)
+		? ",si" : "", (S_ISREG(attr->st_mode)
+		&& (intmax_t)(attr->st_blocks * S_BLKSIZE) < (intmax_t)attr->st_size)
+		? ",sparse" : "");
+}
 
 static void
-print_file_mime(const char *name)
+print_dir_info(const char *dir, const int file_perm)
 {
-#ifdef _NO_MAGIC
-	UNUSED(name);
-	return;
-#else
-	fputs("MIME type:\t", stdout);
-	char *n = xmagic(name, 1);
-	if (n) {
-		printf("%s\n", n);
-		free(n);
-	} else {
-		printf("%s%c%s\n", dn_c, UNKNOWN_CHR, df_c);
+	if (file_perm == 0) {
+		printf(_("Total size: \t%s%c%s\n"), dn_c, UNKNOWN_CHR, NC);
+		printf(_("Items:\t\t%s%c%s\n"), dn_c, UNKNOWN_CHR, NC);
+		return;
 	}
-#endif /* _NO_MAGIC */
+
+	struct dir_info_t info = {0};
+
+	fputs(_("Total size: \t"), stdout);
+
+#define SCANNING_MSG "Scanning..."
+	if (term_caps.suggestions == 1) {
+		HIDE_CURSOR;
+		fputs(dn_c, stdout);
+		fputs(SCANNING_MSG, stdout);
+	}
+	fflush(stdout);
+
+	dir_info(dir, 1, &info);
+
+	if (term_caps.suggestions == 1) {
+		MOVE_CURSOR_LEFT((int)sizeof(SCANNING_MSG) - 1);
+		ERASE_TO_RIGHT;
+		UNHIDE_CURSOR;
+		fputs(df_c, stdout);
+		fflush(stdout);
+	}
+
+	char read_err[5 + (MAX_COLOR * 2)]; *read_err = '\0';
+	if (info.status != 0)
+		snprintf(read_err, sizeof(read_err), "%s%c%s", xf_cb, DU_ERR_CHAR, df_c);
+
+	char *cend = conf.colorize == 1 ? df_c : "";
+	char *size_color = (char *)NULL;
+	char sf[MAX_SHADE_LEN]; *sf = '\0';
+	if (conf.colorize == 1 && !*dz_c) {
+		get_color_size(info.size, sf, sizeof(sf));
+		size_color = sf;
+	}
+
+	char *size = construct_human_size(info.size);
+	if (size) {
+		printf("%s%s%s%s ", read_err, size_color, size, cend);
+
+		const int size_mult_factor = xargs.si == 1 ? 1000 : 1024;
+		if (info.size > size_mult_factor)
+			printf("/ %s%jd B%s ", size_color, (intmax_t)info.size, cend);
+
+		printf("(%s%s)\n", conf.apparent_size == 1 ? _("apparent")
+			: _("disk usage"), xargs.si == 1 ? ",si" : "");
+	} else {
+		puts(UNKNOWN_STR);
+	}
+
+	printf(_("Items:\t\t%s%s%llu%s (%s%llu%s %s, %s%llu%s %s, %s%llu%s %s)\n"),
+		read_err, BOLD, info.dirs + info.files + info.links, df_c,
+		BOLD, info.dirs, df_c, info.dirs == 1 ? _("directory") : _("directories"),
+		BOLD, info.files, df_c, info.files == 1 ? _("file") : _("files"),
+		BOLD, info.links, df_c, info.links == 1 ? _("link") : _("links"));
 }
-#endif /* CLIFM_PROP_MIME */
+#endif /* USE_DU1 */
 
 /* Retrieve information for the file named FILENAME in a stat(1)-like fashion.
  * If FOLLOW_LINK is set to 1, in which case we're running the 'pp' command
@@ -1847,7 +1886,14 @@ do_stat(char *filename, const int follow_link)
 	char *ctype = dn_c; /* Color for file type char. */
 
 	const int file_perm =
+#ifndef __CYGWIN__
 		check_file_access(attr.st_mode, attr.st_uid, attr.st_gid);
+#else
+		/* check_file_access() checks Unix-style permissions, which does
+		 * not make much sense on Windows file systems (particularly, on
+		 * system directories). */
+		1;
+#endif /* !__CYGWIN__ */
 
 	char *color = get_file_type_and_color(*link_target ? link_target : filename,
 		&attr, &file_type, &ctype);
@@ -1862,16 +1908,21 @@ do_stat(char *filename, const int follow_link)
 	print_file_perms(&attr, file_type, ctype, xattr);
 	print_file_name(filename, color, follow_link, attr.st_mode, link_target);
 	print_file_details(filename, &attr, file_type, file_perm, xattr);
-#ifdef CLIFM_PROP_MIME
 	print_file_mime(*link_target ? link_target : filename);
-#endif
 	print_timestamps(*link_target ? link_target : filename, &attr);
-	print_file_size(filename, &attr, file_perm, follow_link);
 
-#ifdef CLIFM_DIR_INFO
+#ifdef USE_DU1
+	print_file_size(filename, &attr, file_perm, follow_link);
 	if (S_ISDIR(attr.st_mode) && follow_link == 1)
 		print_dir_items(*link_target ? link_target : filename, file_perm);
-#endif /* CLIFM_DIR_INFO */
+#else
+	if (S_ISDIR(attr.st_mode)) {
+		if (follow_link == 1)
+			print_dir_info(*link_target ? link_target : filename, file_perm);
+	} else {
+		print_file_size(&attr);
+	}
+#endif /* USE_DU1 */
 
 	return FUNC_SUCCESS;
 }
@@ -1906,8 +1957,7 @@ properties_function(char **args, const int follow_link)
 	return exit_status;
 }
 
-/* Print properties of the file specified by the global variable STAT_FILENAME
- * and exit.
+/* Print properties of the files passed from the command line.
  * If FULL_STAT is set to 1, run with 'pp'. Otherwise use 'p' instead.
  * Used when running with either --stat or --stat-full. */
 __attribute__ ((noreturn))
@@ -1915,12 +1965,12 @@ void
 do_stat_and_exit(const int full_stat)
 {
 	/* This function is called very early from main(), immediately after loading
-	 * settings, so that we need to set up a few things needed to run the
+	 * settings, so that we need to set up a few things required to run the
 	 * properties function (do_stat). Namely:
 	 * 1. du(1) binary (or xdu).
 	 * 2. tmp dir (if not xdu).
 	 * 3. Current directory. */
-#ifndef USE_XDU
+#ifdef USE_DU1
 # if defined(HAVE_GNU_DU)
 	bin_flags |= GNU_DU_BIN_DU;
 # elif !defined(_BE_POSIX)
@@ -1931,7 +1981,7 @@ do_stat_and_exit(const int full_stat)
 
 	if (!tmp_dir)
 		tmp_dir = savestring(P_tmpdir, P_tmpdir_len);
-#endif /* !USE_XDU */
+#endif /* USE_DU1 */
 
 	fputs(df_c, stdout);
 	cur_ws = 0;
