@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -27,14 +27,13 @@
 
 __BEGIN_DECLS
 
-int  alt_rl_prompt(const char *, const char *);
 int  initialize_readline(void);
-int  is_quote_char(const char);
-char **my_rl_completion(const char *, int, int);
-char *my_rl_path_completion(const char *, int);
-int  rl_get_y_or_n(const char *);
-char *rl_no_hist(const char *);
-int  tab_complete(int);
+int  is_quote_char(const char c);
+char **my_rl_completion(const char *text, int start, int end);
+char *my_rl_path_completion(const char *text, int state);
+int  rl_get_y_or_n(const char *_msg);
+char *rl_no_hist(const char *prompt);
+char *secondary_prompt(const char *_prompt, const char *line);
 
 __END_DECLS
 
