@@ -55,6 +55,7 @@
 #include "readline.h"
 #include "selection.h"
 #include "sort.h"
+#include "xdu.h" /* dir_size() */
 
 /* Save selected elements into a tmp file. Returns 1 on success or 0
  * on error. This function allows the user to work with multiple
@@ -905,9 +906,9 @@ list_selected_files(void)
 	tab_offset = t;
 
 	char *human_size = construct_human_size(total);
-	char err[sizeof(xf_c) + 6]; *err = '\0';
+	char err[sizeof(xf_cb) + 6]; *err = '\0';
 	if (status != 0)
-		snprintf(err, sizeof(err), "%s%c%s", xf_c, DU_ERR_CHAR, NC);
+		snprintf(err, sizeof(err), "%s%c%s", xf_cb, DU_ERR_CHAR, NC);
 
 	char s[MAX_SHADE_LEN]; *s = '\0';
 	if (conf.colorize == 1)
