@@ -749,13 +749,18 @@ clear_fzf(void)
 }
 
 /* Calculate the available space for the fzf preview window based on
- * the main window width, terminal columns, and longest entry.
+ * the main window width, terminal columns, and longest entry (including the
+ * border type).
  * Return (size_t)-1 if the space is less than 50% of total space. */
 static size_t
 get_preview_win_width(const int offset)
 {
 	size_t w = 0;
-	size_t l = longest_prev_entry + 8;
+	size_t l = longest_prev_entry + 8 + (fzf_border_type <= 0 ? 0
+		/* fzf_border_type == 1 (right or left): this takes 2 extra columns.
+		 * fzf_border_type == 2 (rounded, sharp, bold, block, thinblock,
+		 * double, or vertical): this takes 4 extra columns. */
+		: (fzf_border_type == 1 ? 2 : 4));
 	int total_win_width = term_cols - offset;
 
 	if (l < (size_t)total_win_width)
@@ -1514,6 +1519,8 @@ get_finder_offset(const char *query, const char *text, char **matches,
 
 	else if (ct == TCMP_DESEL && query) {
 		finder_offset = prompt_offset + (int)(query - lb) - 3;
+		if (!*query && finder_offset > 0)
+			finder_offset--;
 	}
 
 	else if (ct == TCMP_HIST) {
@@ -2148,6 +2155,14 @@ AFTER_USUAL_COMPLETION:
 		should_quote = matches[0] && rl_completer_quote_characters &&
 		rl_filename_completion_desired && rl_filename_quoting_desired;
 
+//////////
+		/* WORKAROUND: If 'ds' and the replacement string needs to be
+		 * quoted, the completion do not work as expected. */
+		if (cur_comp_type == TCMP_DESEL && matches[0]
+		&& rl_strpbrk(matches[0], quote_chars))
+			replacement = NULL;
+//////////
+
 		if (should_quote)
 			should_quote = (should_quote && !quote_char);
 
@@ -2270,7 +2285,7 @@ AFTER_USUAL_COMPLETION:
 					fputs(cur_color, stdout);
 			} else {
 				char *q = (*replacement == '\\' && *(replacement + 1) == '~')
-						? replacement + 1 : replacement;
+					? replacement + 1 : replacement;
 				rl_insert_text(q);
 				rl_redisplay();
 			}
