@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -34,28 +34,28 @@
 
 #include "aux.h"
 #include "colors.h" /* colors_list() */
-#include "misc.h"
-#include "exec.h"
 #include "init.h"
 #include "messages.h"
+#include "misc.h"
 #include "sort.h"
+#include "spawn.h"
 
 /* A few printing functions */
 static int
 print_tag_creation_error(const char *link, const mode_t mode)
 {
 	if (S_ISLNK(mode))
-		xerror(_("tag: %s: File already tagged\n"), link);
+		xerror(_("tag: '%s': File already tagged\n"), link);
 	else
-		xerror(_("tag: %s: Cannot create tag: file already exists\n"), link);
+		xerror(_("tag: '%s': Cannot create tag: file already exists\n"), link);
 
-	return EXIT_FAILURE;
+	return FUNC_FAILURE;
 }
 
 static int
 print_symlink_error(const char *name)
 {
-	xerror("tag: %s: %s\n", name, strerror(errno));
+	xerror("tag: '%s': %s\n", name, strerror(errno));
 	return errno;
 }
 
@@ -64,14 +64,14 @@ print_no_tags(void)
 {
 	printf(_("%s: No tags found, Use 'tag new' to create new "
 		"tags\n"), PROGRAM_NAME);
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 static int
 print_no_such_tag(const char *name)
 {
-	xerror(_("tag: %s: No such tag\n"), name);
-	return EXIT_FAILURE;
+	xerror(_("tag: '%s': No such tag\n"), name);
+	return FUNC_FAILURE;
 }
 
 static int
@@ -83,12 +83,12 @@ print_usage(const int retval)
 
 #ifndef _DIRENT_HAVE_D_TPYE
 /* Check whether NAME is actually tagged as TAG.
- * Returns 1 if true and zero otherwise. */
+ * Returns 1 if true or 0 otherwise. */
 static int
 check_tagged_file(const char *tag, const char *name)
 {
 	struct stat a;
-	char tmp[PATH_MAX];
+	char tmp[PATH_MAX + 1];
 	snprintf(tmp, sizeof(tmp), "%s/%s/%s", tags_dir, tag, name);
 
 	if (SELFORPARENT(name) || lstat(tmp, &a) == -1 || !S_ISLNK(a.st_mode))
@@ -102,8 +102,10 @@ check_tagged_file(const char *tag, const char *name)
 static void
 print_tagged_file(char *name, const char *tag)
 {
-	char dir[PATH_MAX], tmp[PATH_MAX];
+	char dir[PATH_MAX + 1];
+	char tmp[PATH_MAX + 1];
 	*tmp = '\0';
+
 	snprintf(dir, sizeof(dir), "%s/%s/%s", tags_dir, tag, name);
 	char *ret = realpath(dir, tmp);
 	if (!ret)
@@ -120,7 +122,7 @@ print_tagged_file(char *name, const char *tag)
 		q = home_tilde(tmp, &free_name);
 
 	if (strchr(name, '\\')) {
-		char *d = dequote_str(name, 0);
+		char *d = unescape_str(name, 0);
 		if (d) {
 			xstrsncpy(name, d, strlen(d) + 1);
 			free(d);
@@ -142,28 +144,30 @@ static int
 list_files_in_tag(char *name)
 {
 	if (strchr(name, '\\')) {
-		char *p = dequote_str(name, 0);
+		char *p = unescape_str(name, 0);
 		if (p) {
 			xstrsncpy(name, p, strlen(p) + 1);
 			free(p);
 		}
 	}
 
-	char tmp[PATH_MAX];
+	char tmp[PATH_MAX + 1];
 	snprintf(tmp, sizeof(tmp), "%s/%s", tags_dir, name);
 
 	struct dirent **t = (struct dirent **)NULL;
-	int n = scandir(tmp, &t, NULL, conf.case_sens_list
+	const int n = scandir(tmp, &t, NULL, conf.case_sens_list
 		? xalphasort : alphasort_insensitive);
-	if (n == -1)
+	if (n == -1) {
+		xerror("tag: '%s': %s\n", tmp, strerror(errno));
 		return errno;
+	}
 
 	size_t i;
 	if (n <= 2) {
 		for (i = 0; i < (size_t)n; i++)
 			free(t[i]);
 		free(t);
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	for (i = 0; i < (size_t)n; i++) {
@@ -180,7 +184,7 @@ list_files_in_tag(char *name)
 	}
 	free(t);
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Return the length of longest tag name.
@@ -191,7 +195,7 @@ get_longest_tag(void)
 	size_t i, _longest = 0;
 
 	for (i = 0; i < tags_n; i++) {
-		size_t l = strlen(tags[i]);
+		const size_t l = strlen(tags[i]);
 		if (l > (size_t)_longest)
 			_longest = l;
 	}
@@ -209,7 +213,7 @@ list_tags_having_file(const dev_t dev, const ino_t ino)
 
 	size_t i;
 	for (i = 0; tags[i]; i++) {
-		char tmp[PATH_MAX];
+		char tmp[PATH_MAX + 1];
 		snprintf(tmp, sizeof(tmp), "%s/%s", tags_dir, tags[i]);
 
 		DIR *dir = opendir(tmp);
@@ -244,7 +248,7 @@ is_tag(char *name)
 		return 0;
 
 	if (strchr(name, '\\')) {
-		char *deq = dequote_str(name, 0);
+		char *deq = unescape_str(name, 0);
 		if (deq) {
 			xstrsncpy(name, deq, strlen(deq) + 1);
 			free(deq);
@@ -266,17 +270,17 @@ list_tags_full(void)
 {
 	if (tags_n == 0) {
 		puts(_("tag: No tags"));
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	size_t i;
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 
 	for (i = 0; tags[i]; i++) {
-		printf(_("Files tagged as %s%s%s:\n"), conf.colorize == 1 ? BOLD : "'",
-			tags[i], conf.colorize == 1 ? NC : "'");
-		if (list_files_in_tag(tags[i]) != EXIT_SUCCESS)
-			exit_status = EXIT_FAILURE;
+		printf(_("Files tagged as %s%s%s:\n"), conf.colorize == 1
+			? BOLD : "'", tags[i], conf.colorize == 1 ? NC : "'");
+		if (list_files_in_tag(tags[i]) != FUNC_SUCCESS)
+			exit_status = FUNC_FAILURE;
 
 		if (tags[i + 1])
 			putchar('\n');
@@ -292,16 +296,16 @@ list_tags(char **args)
 		return print_no_tags();
 
 	size_t i;
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 
 	if (!args || !args[0] || !args[1] || !args[2]) {
 		/* 'tag list': list all tags */
-		int pad = (int)get_longest_tag();
+		const int pad = (int)get_longest_tag();
 
 		for (i = 0; tags[i]; i++) {
-			char p[PATH_MAX];
+			char p[PATH_MAX + 1];
 			snprintf(p, sizeof(p), "%s/%s", tags_dir, tags[i]);
-			filesn_t n = count_dir(p, NO_CPOP);
+			const filesn_t n = count_dir(p, NO_CPOP);
 			if (n > 2)
 				printf("%-*s [%s%jd%s]\n", pad, tags[i], mi_c,
 					(intmax_t)n - 2, df_c);
@@ -309,13 +313,13 @@ list_tags(char **args)
 				printf("%-*s  -\n", pad, tags[i]);
 		}
 
-		return EXIT_SUCCESS;
+		return FUNC_SUCCESS;
 	}
 
 	for (i = 2; args[i]; i++) {
 		if (!is_tag(args[i])) {
 			/* 'tag list FILENAME' */
-			char *p = dequote_str(args[i], 0);
+			char *p = unescape_str(args[i], 0);
 
 			struct stat a;
 			if (lstat(p ? p : args[i], &a) == -1) {
@@ -332,10 +336,11 @@ list_tags(char **args)
 
 		} else {
 			/* 'tag list TAG' */
-			printf(_("Files tagged as %s%s%s:\n"), conf.colorize == 1 ? BOLD : "'",
-				args[i], conf.colorize == 1 ? BOLD : "'");
-			if (list_files_in_tag(args[i]) == EXIT_FAILURE)
-				exit_status = EXIT_FAILURE;
+			printf(_("Files tagged as %s%s%s%s:\n"), conf.colorize == 1 ? BOLD : "'",
+				args[i], conf.colorize == 1 ? BOLD : "'",
+				conf.colorize == 1 ? NC : "'");
+			if (list_files_in_tag(args[i]) == FUNC_FAILURE)
+				exit_status = FUNC_FAILURE;
 		}
 
 		if (args[i + 1])
@@ -357,15 +362,16 @@ static int
 create_tags(char **args)
 {
 	if (!args || !args[1] || !args[2])
-		return print_usage(EXIT_FAILURE);
+		return print_usage(FUNC_FAILURE);
 
 	size_t i;
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 
 	for (i = 2; args[i]; i++) {
-		char *p = strchr(args[i], '\\'), dir[PATH_MAX];
+		char dir[PATH_MAX + 1];
+		char *p = strchr(args[i], '\\');
 		if (p) {
-			char *deq = dequote_str(args[i], 0);
+			char *deq = unescape_str(args[i], 0);
 			if (deq) {
 				free(args[i]);
 				args[i] = deq;
@@ -376,15 +382,15 @@ create_tags(char **args)
 
 		struct stat a;
 		if (lstat(dir, &a) != -1) {
-			xerror(_("tag: %s: Tag already exists\n"), args[i]);
-			exit_status = EXIT_FAILURE;
+			xerror(_("tag: '%s': Tag already exists\n"), args[i]);
+			exit_status = FUNC_FAILURE;
 			continue;
 		}
 
-		if (xmkdir(dir, S_IRWXU) == EXIT_FAILURE) {
-			xerror(_("tag: %s: Error creating tag: %s\n"),
+		if (xmkdir(dir, S_IRWXU) == FUNC_FAILURE) {
+			xerror(_("tag: '%s': Error creating tag: %s\n"),
 				args[i], strerror(errno));
-			exit_status = EXIT_FAILURE;
+			exit_status = FUNC_FAILURE;
 			continue;
 		}
 
@@ -402,19 +408,19 @@ remove_tags(char **args)
 	if (tags_n == 0)
 		return print_no_tags();
 
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 	size_t i;
 	for (i = 2; args[i]; i++) {
 		char *p = strchr(args[i], '\\');
 		if (p) {
-			char *deq = dequote_str(args[i], 0);
+			char *deq = unescape_str(args[i], 0);
 			if (deq) {
 				free(args[i]);
 				args[i] = deq;
 			}
 		}
 
-		char dir[PATH_MAX];
+		char dir[PATH_MAX + 1];
 		snprintf(dir, sizeof(dir), "%s/%s", tags_dir, args[i]);
 
 		struct stat a;
@@ -422,11 +428,11 @@ remove_tags(char **args)
 			return print_no_such_tag(args[i]);
 
 		char *cmd[] = {"rm", "-r", "--", dir, NULL};
-		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) == EXIT_SUCCESS) {
-			printf(_("%s: Successfully removed tag\n"), args[i]);
+		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) == FUNC_SUCCESS) {
+			printf(_("'%s': Successfully removed tag\n"), args[i]);
 			reload_tags();
 		} else {
-			exit_status = EXIT_FAILURE;
+			exit_status = FUNC_FAILURE;
 		}
 	}
 
@@ -439,23 +445,23 @@ tag_file(char *name, char *tag)
 {
 	struct stat a;
 	if (lstat(name, &a) == -1) {
-		xerror("tag: %s: %s\n", name, strerror(errno));
-		return EXIT_FAILURE;
+		xerror("tag: '%s': %s\n", name, strerror(errno));
+		return FUNC_FAILURE;
 	}
 
 	int new_tag = 0;
 	char *p = (char *)NULL;
 	if (strchr(tag, '\\'))
-		p = dequote_str(tag, 0);
-	char dir[PATH_MAX];
+		p = unescape_str(tag, 0);
+	char dir[PATH_MAX + 1];
 	snprintf(dir, sizeof(dir), "%s/%s", tags_dir, p ? p : tag);
 
 	if (stat(dir, &a) == -1) {
-		if (xmkdir(dir, S_IRWXU) != EXIT_SUCCESS) {
-			xerror(_("tag: %s: Cannot create tag: %s\n"),
+		if (xmkdir(dir, S_IRWXU) != FUNC_SUCCESS) {
+			xerror(_("tag: '%s': Cannot create tag: %s\n"),
 				p ? p : tag, strerror(errno));
 			free(p);
-			return EXIT_FAILURE;
+			return FUNC_FAILURE;
 		}
 		new_tag = 1;
 	}
@@ -467,7 +473,7 @@ tag_file(char *name, char *tag)
 	}
 	free(p);
 
-	char name_path[PATH_MAX];
+	char name_path[PATH_MAX + 1];
 	*name_path = '\0';
 	if (*name != '/') {
 		snprintf(name_path, sizeof(name_path), "%s/%s",
@@ -486,7 +492,7 @@ tag_file(char *name, char *tag)
 	if (symlink(*name_path ? name_path : name, link) == -1)
 		return print_symlink_error(name);
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Return an array with indices of tag names (:TAG) found in the ARGS array.
@@ -498,7 +504,7 @@ get_tags(char **args)
 	int n, i, c = 0;
 
 	for (n = 0; args[n]; n++);
-	int *t = (int *)xnmalloc((size_t)n + 1, sizeof(int));
+	int *t = xnmalloc((size_t)n + 1, sizeof(int));
 
 	for (i = 0; i < n; i++) {
 		if (*args[i] == ':' && *(args[i] + 1)) {
@@ -526,11 +532,11 @@ tag_files(char **args)
 		free(tag_names);
 		xerror("%s\n", _("tag: No tag specified. Specify a tag via :TAG. "
 			"E.g. 'tag add FILE1 FILE2 :TAG'"));
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	size_t i, j, n = 0;
-	size_t start = (args[1] && strcmp(args[1], "add") == 0) ? 2 : 1;
+	const size_t start = (args[1] && strcmp(args[1], "add") == 0) ? 2 : 1;
 
 	for (i = start; args[i]; i++) {
 		if (*args[i] != ':')
@@ -544,9 +550,9 @@ tag_files(char **args)
 
 			char *p = (char *)NULL;
 			if (strchr(args[j], '\\'))
-				p = dequote_str(args[j], 0);
+				p = unescape_str(args[j], 0);
 
-			if (tag_file(p ? p : args[j], args[tag_names[i]] + 1) != EXIT_SUCCESS)
+			if (tag_file(p ? p : args[j], args[tag_names[i]] + 1) != FUNC_SUCCESS)
 				if (n > 0) --n;
 			free(p);
 		}
@@ -556,7 +562,7 @@ tag_files(char **args)
 
 	if (n > 0)
 		printf(_("Successfully tagged %zu file(s)\n"), n);
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Untag file names found in ARGS tagged as ARGS[N]. */
@@ -564,16 +570,16 @@ static int
 untag(char **args, const size_t n, size_t *t)
 {
 	if (!args || !args[1])
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 	size_t i;
 	for (i = 2; args[i]; i++) {
 		if (i == n || (*args[i] == ':' && *(args[1] + 1)))
 			continue;
 
-		char *ds = dequote_str(args[n] + 1, 0);
-		char dir[PATH_MAX];
+		char *ds = unescape_str(args[n] + 1, 0);
+		char dir[PATH_MAX + 1];
 		snprintf(dir, sizeof(dir), "%s/%s", tags_dir, ds ? ds : args[n] + 1);
 		free(ds);
 
@@ -582,7 +588,7 @@ untag(char **args, const size_t n, size_t *t)
 			return print_no_such_tag(args[n] + 1);
 
 		char f[PATH_MAX + NAME_MAX];
-		char *deq = dequote_str(args[i], 0);
+		char *deq = unescape_str(args[i], 0);
 		char *p = deq ? deq : args[i];
 		char *exp = (char *)NULL;
 		if (*p == '~')
@@ -599,12 +605,12 @@ untag(char **args, const size_t n, size_t *t)
 			errno = 0;
 			if (unlinkat(XAT_FDCWD, f, 0) == -1) {
 				exit_status = errno;
-				xerror("tag: %s: %s\n", args[i], strerror(errno));
+				xerror("tag: '%s': %s\n", args[i], strerror(errno));
 			} else {
 				(*t)++;
 			}
 		} else {
-			xerror(_("tag: %s: File not tagged as %s%s%s\n"),
+			xerror(_("tag: '%s': File not tagged as %s%s%s\n"),
 				args[i], conf.colorize ? BOLD : "", args[n] + 1, df_c);
 			continue;
 		}
@@ -617,13 +623,13 @@ untag(char **args, const size_t n, size_t *t)
 static int
 untag_files(char **args)
 {
-	int exit_status = EXIT_SUCCESS;
+	int exit_status = FUNC_SUCCESS;
 	size_t i, n = 0;
 
 	for (i = 1; args[i]; i++) {
 		if (*args[i] == ':' && *(args[i] + 1)
-		&& untag(args, i, &n) == EXIT_FAILURE)
-			exit_status = EXIT_FAILURE;
+		&& untag(args, i, &n) == FUNC_FAILURE)
+			exit_status = FUNC_FAILURE;
 	}
 
 	if (n > 0)
@@ -637,7 +643,7 @@ static int
 rename_tag(char **args)
 {
 	if (!args || !args[1] || !args[2] || !args[3])
-		return print_usage(EXIT_FAILURE);
+		return print_usage(FUNC_FAILURE);
 
 	char *old = args[2], *new = args[3];
 	if (!is_tag(old))
@@ -645,13 +651,13 @@ rename_tag(char **args)
 
 	if (*old == *new && strcmp(old, new) == 0) {
 		xerror("%s\n", "tag: New and old file names are the same");
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
-	char old_dir[PATH_MAX];
+	char old_dir[PATH_MAX + 1];
 	snprintf(old_dir, sizeof(old_dir), "%s/%s", tags_dir, old);
 
-	char new_dir[PATH_MAX];
+	char new_dir[PATH_MAX + 1];
 	snprintf(new_dir, sizeof(new_dir), "%s/%s", tags_dir, new);
 
 	if (rename(old_dir, new_dir) == -1) {
@@ -661,7 +667,7 @@ rename_tag(char **args)
 
 	puts("Successfully renamed tag");
 	reload_tags();
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Move all (tagged) files (symlinks) in SRC into DST.
@@ -669,15 +675,16 @@ rename_tag(char **args)
 static int
 recursive_mv_tags(const char *src, const char *dst)
 {
-	int i, n, exit_status = EXIT_SUCCESS, ret;
-	char src_dir[PATH_MAX], dst_dir[PATH_MAX];
+	int i, n, exit_status = FUNC_SUCCESS, ret;
+	char src_dir[PATH_MAX + 1];
+	char dst_dir[PATH_MAX + 1];
 	struct dirent **a = (struct dirent **)NULL;
 
 	snprintf(src_dir, sizeof(src_dir), "%s/%s", tags_dir, src);
 
 	n = scandir(src_dir, &a, NULL, alphasort);
 	if (n == -1) {
-		xerror("tag: %s: %s\n", src_dir, strerror(errno));
+		xerror("tag: '%s': %s\n", src_dir, strerror(errno));
 		return errno;
 	}
 
@@ -691,7 +698,7 @@ recursive_mv_tags(const char *src, const char *dst)
 		char src_file[PATH_MAX + NAME_MAX + 2];
 		snprintf(src_file, sizeof(src_file), "%s/%s", src_dir, a[i]->d_name);
 		char *cmd[] = {"mv", "--", src_file, dst_dir, NULL};
-		if ((ret = launch_execv(cmd, FOREGROUND, E_NOFLAG)) != EXIT_SUCCESS)
+		if ((ret = launch_execv(cmd, FOREGROUND, E_NOFLAG)) != FUNC_SUCCESS)
 			exit_status = ret;
 		free(a[i]);
 	}
@@ -705,7 +712,7 @@ static int
 merge_tags(char **args)
 {
 	if (!args || !args[1] || !args[2] || !args[3])
-		return print_usage(EXIT_FAILURE);
+		return print_usage(FUNC_FAILURE);
 
 	if (!is_tag(args[2]))
 		return print_no_such_tag(args[2]);
@@ -716,20 +723,20 @@ merge_tags(char **args)
 
 	if (strcmp(src, dst) == 0) {
 		xerror("%s\n", _("tag: Source and destiny are the same tag"));
-		return EXIT_FAILURE;
+		return FUNC_FAILURE;
 	}
 
 	errno = 0;
 	int exit_status = recursive_mv_tags(src, dst);
-	if (exit_status != EXIT_SUCCESS) {
+	if (exit_status != FUNC_SUCCESS) {
 		xerror("%s\n", _("tag: Cannot merge tags: error moving tagged files"));
 		return exit_status;
 	}
 
-	char src_dir[PATH_MAX];
+	char src_dir[PATH_MAX + 1];
 	snprintf(src_dir, sizeof(src_dir), "%s/%s", tags_dir, src);
 	if (rmdir(src_dir) == -1) {
-		xerror("tag: %s: %s\n", src_dir, strerror(errno));
+		xerror("tag: '%s': %s\n", src_dir, strerror(errno));
 		return errno;
 	}
 
@@ -738,7 +745,7 @@ merge_tags(char **args)
 		conf.colorize == 1 ? BOLD : "", src, df_c,
 		conf.colorize == 1 ? BOLD : "", dst, df_c);
 
-	return EXIT_SUCCESS;
+	return FUNC_SUCCESS;
 }
 
 /* Perform the following expansions:
@@ -756,7 +763,7 @@ reconstruct_input(char **args)
 {
 	size_t n = 0, c;
 	for (n = 0; args[n]; n++);
-	char **a = (char **)xnmalloc(n + 2, sizeof(char *));
+	char **a = xnmalloc(n + 2, sizeof(char *));
 
 	a[0] = savestring("tag", 3);
 
@@ -811,7 +818,7 @@ is_tag_help(char **args)
 int
 tags_function(char **args)
 {
-	int exit_status = EXIT_SUCCESS, free_args = 0;
+	int exit_status = FUNC_SUCCESS, free_args = 0;
 	char **a = args;
 
 	if (is_tag_help(a))
