@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
+ * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -27,8 +27,8 @@
 
 __BEGIN_DECLS
 
-int is_tag(char *);
-int tags_function(char **);
+int is_tag(char *name);
+int tags_function(char **args);
 
 __END_DECLS
 
