@@ -43,6 +43,7 @@ xatoi, url_encode, xnmalloc */
 #include "readline.h"   /* rl_no_hist, rl_get_y_or_n */
 #include "sort.h"       /* skip_files, xalphasort, alphasort_insensitive */
 #include "spawn.h"      /* launch_execv */
+#include "xdu.h"        /* dir_size */
 
 /* Return the amount of currently trashed files. */
 static size_t
@@ -932,9 +933,9 @@ print_trashdir_size(void)
 	const off_t full_size = dir_size(trash_files_dir, 0, &status) * base;
 	char *human_size = construct_human_size(full_size);
 
-	char err[sizeof(xf_c) + 6]; *err = '\0';
+	char err[sizeof(xf_cb) + 6]; *err = '\0';
 	if (status != 0)
-		snprintf(err, sizeof(err), "%s%c%s", xf_c, DU_ERR_CHAR, NC);
+		snprintf(err, sizeof(err), "%s%c%s", xf_cb, DU_ERR_CHAR, NC);
 
 	char s[MAX_SHADE_LEN]; *s = '\0';
 	if (conf.colorize == 1)
